/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Blitter Source (Independent)
* Author  : Dave Reed
*/

/*-Project-Includes----------------------------------------------------------*/

#include "Main.h"
#include "Base/Render.h"
#include "Base/Blitter.h"


/*- 2D Blitter Class -------------------------------------------------------*/

/*-Class-Static-Variables----------------------------------------------------*/

Vertex2D		  Blitter2D::s_axVerts[BLITTER2D_BATCH_SIZE*4];
int32		 	  Blitter2D::s_aiIndices[BLITTER2D_BATCH_SIZE*6];
Vertex2D		  Blitter2D::s_axLineVerts[BLITTER2D_LINE_BATCH_SIZE*2];
int32			  Blitter2D::s_iNumBatchedLines	= 0;

Vertex3D		  Blitter3D::s_axVerts[BLITTER3D_BATCH_SIZE*4];
int32		 	  Blitter3D::s_aiIndices[BLITTER3D_BATCH_SIZE*6];
Vertex3D		  Blitter3D::s_axLineVerts[BLITTER3D_LINE_BATCH_SIZE*2];
int32			  Blitter3D::s_iNumBatchedLines	= 0;


/*-Class-Public-Functions----------------------------------------------------*/

void Blitter2D::Initialize()
{
	for(int i=0;i<BLITTER2D_BATCH_SIZE;i++)
	{
		s_aiIndices[(i*6)+0] = (i*4)+0;
		s_aiIndices[(i*6)+1] = (i*4)+1;
		s_aiIndices[(i*6)+2] = (i*4)+2;
		s_aiIndices[(i*6)+3] = (i*4)+2;
		s_aiIndices[(i*6)+4] = (i*4)+3;
		s_aiIndices[(i*6)+5] = (i*4)+0;
	}	

	for(int i=0;i<BLITTER2D_LINE_BATCH_SIZE;i++)
	{
		s_axLineVerts[(i*2)+0].fU = 0;
		s_axLineVerts[(i*2)+0].fV = 0;		
		s_axLineVerts[(i*2)+1].fU = 0;			
		s_axLineVerts[(i*2)+1].fV = 0;
	}	
}

void Blitter2D::Shutdown()
{
	
}



void Blitter2D::BlitRects(int iCount, BlitRect2D *pxRects, Texture *pxTexture, SHADER eShader)
{
	if (iCount<=0) return;

	// If the batch is too big, recursively subdivide
	if (iCount>BLITTER2D_BATCH_SIZE)
	{
		BlitRects(BLITTER2D_BATCH_SIZE, pxRects, pxTexture, eShader);
		BlitRects(iCount - BLITTER2D_BATCH_SIZE, &pxRects[BLITTER2D_BATCH_SIZE], pxTexture, eShader);
	}

	// Set the texture in the current shader
	Shaders::SetTexture(eShader, 0, pxTexture?pxTexture:Texture::GetBaseTexture(BASE_TEXTURE_WHITE));	

	// Create the verts
	int iVert=0;
	BlitRect2D *pxRect=&pxRects[0];
	for(int i=0;i<iCount;i++)
	{
		// TopLeft
		s_axVerts[iVert].fX	   = pxRect->fX[0];
		s_axVerts[iVert].fY	   = pxRect->fY[0];
		s_axVerts[iVert].fU	   = pxRect->fU[0];
		s_axVerts[iVert].fV	   = pxRect->fV[0];
		s_axVerts[iVert].uARGB = pxRect->uARGB;
		iVert++;

		// TopRight
		s_axVerts[iVert].fX	   = pxRect->fX[1];
		s_axVerts[iVert].fY	   = pxRect->fY[0];
		s_axVerts[iVert].fU	   = pxRect->fU[1];
		s_axVerts[iVert].fV	   = pxRect->fV[0];
		s_axVerts[iVert].uARGB = pxRect->uARGB;
		iVert++;

		// BotRight
		s_axVerts[iVert].fX	   = pxRect->fX[1];
		s_axVerts[iVert].fY	   = pxRect->fY[1];
		s_axVerts[iVert].fU	   = pxRect->fU[1];
		s_axVerts[iVert].fV	   = pxRect->fV[1];
		s_axVerts[iVert].uARGB = pxRect->uARGB;	
		iVert++;

		// BotLeft
		s_axVerts[iVert].fX	   = pxRect->fX[0];
		s_axVerts[iVert].fY	   = pxRect->fY[1];
		s_axVerts[iVert].fU	   = pxRect->fU[0];
		s_axVerts[iVert].fV	   = pxRect->fV[1];
		s_axVerts[iVert].uARGB = pxRect->uARGB;
		iVert++;

		pxRect++;	
	}

	// Finally, do the render calls
	Platform_BlitRects(iCount, eShader);
}


void Blitter2D::DrawLine(float X0, float Y0, float X1, float Y1, uint32 uARGB)
{
	if (s_iNumBatchedLines>=BLITTER2D_LINE_BATCH_SIZE)
	{
		Flush();
	}

	int i = s_iNumBatchedLines * 2;

	s_axLineVerts[i+0].fX	 = X0;
	s_axLineVerts[i+0].fY	 = Y0;
	s_axLineVerts[i+0].uARGB = uARGB;

	s_axLineVerts[i+1].fX	 = X1;
	s_axLineVerts[i+1].fY	 = Y1;
	s_axLineVerts[i+1].uARGB = uARGB;

	s_iNumBatchedLines++;
}


void Blitter2D::FlushLines()
{
	if (s_iNumBatchedLines>0)
	{		
		Shaders::SetTexture(SHADER_BLITTER2D, 0, Texture::GetBaseTexture(BASE_TEXTURE_WHITE));	
		Platform_RenderLines(s_iNumBatchedLines);
	}

	s_iNumBatchedLines = 0;
}


void Blitter2D::Flush()
{
	FlushLines();
}






void Blitter3D::Initialize()
{
	for(int i=0;i<BLITTER3D_BATCH_SIZE;i++)
	{
		s_aiIndices[(i*6)+0] = (i*4)+0;
		s_aiIndices[(i*6)+1] = (i*4)+1;
		s_aiIndices[(i*6)+2] = (i*4)+2;
		s_aiIndices[(i*6)+3] = (i*4)+2;
		s_aiIndices[(i*6)+4] = (i*4)+3;
		s_aiIndices[(i*6)+5] = (i*4)+0;
	}	

	for(int i=0;i<BLITTER3D_LINE_BATCH_SIZE;i++)
	{
		s_axLineVerts[(i*2)+0].fU = 0;
		s_axLineVerts[(i*2)+0].fV = 0;		
		s_axLineVerts[(i*2)+1].fU = 0;			
		s_axLineVerts[(i*2)+1].fV = 0;
	}
}

void Blitter3D::Shutdown()
{

}

void Blitter3D::BlitQuads(int iCount, BlitQuad3D *pxQuads, Texture *pxTexture, SHADER eShader)
{
	if (iCount<=0) return;

	// If the batch is too big, recursively subdivide
	if (iCount>BLITTER3D_BATCH_SIZE)
	{
		BlitQuads(BLITTER3D_BATCH_SIZE, pxQuads, pxTexture, eShader);
		BlitQuads(iCount - BLITTER3D_BATCH_SIZE, &pxQuads[BLITTER3D_BATCH_SIZE], pxTexture, eShader);
	}


	// Set the texture
	Shaders::SetTexture(eShader, 0, pxTexture?pxTexture:Texture::GetBaseTexture(BASE_TEXTURE_WHITE));	

	// Create the verts
	int iVert=0;
	BlitQuad3D *pxQuad=&pxQuads[0];

	for(int i=0;i<iCount;i++)
	{
		// TopLeft
		s_axVerts[iVert].fX	   = pxQuad->fX[0];
		s_axVerts[iVert].fY	   = pxQuad->fY[0];
		s_axVerts[iVert].fZ	   = pxQuad->fZ[0];
		s_axVerts[iVert].fU	   = pxQuad->fU[0];
		s_axVerts[iVert].fV	   = pxQuad->fV[0];
		s_axVerts[iVert].uARGB = pxQuad->uARGB;
		iVert++;

		// TopRight
		s_axVerts[iVert].fX	   = pxQuad->fX[1];
		s_axVerts[iVert].fY	   = pxQuad->fY[1];
		s_axVerts[iVert].fZ	   = pxQuad->fZ[1];
		s_axVerts[iVert].fU	   = pxQuad->fU[1];
		s_axVerts[iVert].fV	   = pxQuad->fV[1];
		s_axVerts[iVert].uARGB = pxQuad->uARGB;
		iVert++;

		// BotRight
		s_axVerts[iVert].fX	   = pxQuad->fX[2];
		s_axVerts[iVert].fY	   = pxQuad->fY[2];
		s_axVerts[iVert].fZ	   = pxQuad->fZ[2];
		s_axVerts[iVert].fU	   = pxQuad->fU[2];
		s_axVerts[iVert].fV	   = pxQuad->fV[2];
		s_axVerts[iVert].uARGB = pxQuad->uARGB;	
		iVert++;

		// BotLeft
		s_axVerts[iVert].fX	   = pxQuad->fX[3];
		s_axVerts[iVert].fY	   = pxQuad->fY[3];
		s_axVerts[iVert].fZ	   = pxQuad->fZ[3];
		s_axVerts[iVert].fU	   = pxQuad->fU[3];
		s_axVerts[iVert].fV	   = pxQuad->fV[3];
		s_axVerts[iVert].uARGB = pxQuad->uARGB;
		iVert++;

		pxQuad++;	
	}

	// Finally, do the render calls
	Platform_BlitQuads(iCount, eShader);
}


void Blitter3D::DrawLine(Vector &xP1, Vector &xP2, uint32 uARGB)
{
	if (s_iNumBatchedLines>=BLITTER3D_LINE_BATCH_SIZE)
	{
		Flush();
	}

	int i = s_iNumBatchedLines * 2;

	s_axLineVerts[i+0].fX	 = xP1.m_fX;
	s_axLineVerts[i+0].fY	 = xP1.m_fY;
	s_axLineVerts[i+0].fZ	 = xP1.m_fZ;
	s_axLineVerts[i+0].uARGB = uARGB;

	s_axLineVerts[i+1].fX	 = xP2.m_fX;
	s_axLineVerts[i+1].fY	 = xP2.m_fY;
	s_axLineVerts[i+1].fZ	 = xP2.m_fZ;
	s_axLineVerts[i+1].uARGB = uARGB;

	s_iNumBatchedLines++;
}


void Blitter3D::FlushLines()
{
	if (s_iNumBatchedLines>0)
	{		
		Shaders::SetTexture(SHADER_BLITTER3D, 0, Texture::GetBaseTexture(BASE_TEXTURE_WHITE));	
		Platform_RenderLines(s_iNumBatchedLines);
	}
	s_iNumBatchedLines = 0;
}

void Blitter3D::Flush()
{
	FlushLines();
}
