/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Blitter Header (Independent)
* Author  : Dave Reed
*/

/*-Include-Once--------------------------------------------------------------*/

#if !defined(__BASE_BLITTER_H__)
#define __BASE_BLITTER_H__ 

/*-Project-Includes----------------------------------------------------------*/

#include "Base/Texture.h"
#include "Base/Shaders.h"
#include "Platform_Blitter.h"

/*-Defines-------------------------------------------------------------------*/

#define BLITTER2D_BATCH_SIZE	  128
#define BLITTER2D_LINE_BATCH_SIZE 128

#define BLITTER3D_BATCH_SIZE	  128
#define BLITTER3D_LINE_BATCH_SIZE 512

/*-Classes-------------------------------------------------------------------*/

struct BlitRect2D
{
	float	fX[2];
	float	fY[2];		
	float	fU[2];
	float	fV[2];
	uint32  uARGB;
};

struct BlitQuad3D
{
	float	fX[4];
	float	fY[4];
	float	fZ[4];
	float	fU[4];
	float	fV[4];
	uint32  uARGB;
};


class Blitter2D
{
public:
	static void Initialize(void);
	static void Shutdown(void);
	static void BlitRects(int iCount, BlitRect2D *pxRects, Texture *pxTexture, SHADER eShader = SHADER_BLITTER2D);
	static void DrawLine(float X0, float Y0, float X1, float Y1, uint32 uARGB);
	static void FlushLines(void);
	static void Flush(void);

private:

	static void Platform_RenderLines(int iCount);
	static void Platform_BlitRects(int iCount, SHADER eShader);

	static Vertex2D			  s_axVerts[BLITTER2D_BATCH_SIZE*4];
	static int32		 	  s_aiIndices[BLITTER2D_BATCH_SIZE*6];
	static Vertex2D			  s_axLineVerts[BLITTER2D_LINE_BATCH_SIZE*2];
	static int32			  s_iNumBatchedLines;
};




class Blitter3D
{
public:
	static void Initialize(void);
	static void Shutdown(void);
	static void BlitQuads(int iCount, BlitQuad3D *pxRects, Texture *pxTexture, SHADER eShader = SHADER_BLITTER3D);
	static void DrawLine(Vector &xP0, Vector &xP1, uint32 uARGB);
	static void FlushLines(void);
	static void Flush(void);

private:
	static void Platform_RenderLines(int iCount);		
	static void Platform_BlitQuads(int iCount, SHADER eShader);		
	
	static Vertex3D		      s_axVerts[BLITTER3D_BATCH_SIZE*4];		
	static int32		 	  s_aiIndices[BLITTER3D_BATCH_SIZE*6];
	static Vertex3D			  s_axLineVerts[BLITTER3D_LINE_BATCH_SIZE*2];
	static int32			  s_iNumBatchedLines;
};

#endif // __BASE_BLITTER_H__ 