/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Camera Source (Independent)
* Author  : Dave Reed
*/

/*-Project-Includes----------------------------------------------------------*/

#include "Main.h"
#include "Base\Camera.h"
#include "Platform_Render.h"

/*-Class-Public-Functions----------------------------------------------------*/

static Camera s_CurrentCamera;

Camera::Camera()
{
	m_fFOV    = 0.0f;
	m_fAspect = 0.0f;
	m_fNearZ  = 0.0f;
	m_fFarZ   = 0.0f;

	m_xPosition = Vector( 0.0f, 5.0f, -10.0f );
	m_xTarget   = Vector( 0.0f, 0.0f, 0.0f );
	m_xUp	    = Vector( 0.0f, 1.0f, 0.0f );

	m_xMtxProj.SetIdentity();
	m_xMtxView.SetIdentity();
	m_xMtxViewProj.SetIdentity();
}


void Camera::SetUpVector(Vector &xUp)
{
	m_xUp		= xUp;
	UpdateMatrices();
}


void Camera::SetLookAt(Vector &xFrom, Vector &xAt)
{
	m_xPosition  = xFrom;
	m_xTarget	 = xAt;
	UpdateMatrices();
}


void Camera::SetProjection(float fFoV, float fAspect, float fZNear, float fZFar)
{
	m_fFOV	  = fFoV;
	m_fAspect = fAspect;
	m_fNearZ  = fZNear;
	m_fFarZ   = fZFar;
	UpdateMatrices();
}


void Camera::UpdateMatrices(void)
{
	m_xMtxView.SetLookAtMatrix(m_xPosition, m_xTarget, m_xUp);
	m_xMtxProj.SetPerspectiveMatrix(m_fFOV, m_fAspect, m_fNearZ, m_fFarZ);
	m_xMtxViewProj = (m_xMtxView * m_xMtxProj);

	m_xFrustum.FromMatrix(m_xMtxViewProj);
}


void Camera::Set(Camera &rxCamera)
{
	s_CurrentCamera = rxCamera;
	Platform_Render::SetCamera(rxCamera);
}


Camera &Camera::GetCurrentCamera(void)
{
	return s_CurrentCamera;
}