
/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Camera Header (Independent)
* Author  : Dave Reed
*/

/*-Include-Once--------------------------------------------------------------*/

#if !defined(__BASE_CAMERA_H__)
#define __BASE_CAMERA_H__ /*!< \brief Include once define. */

/*-Project-Includes----------------------------------------------------------*/

#include "Base/Math.h"

class Camera
{	
public:		

	// Constructor
	Camera();

	// Set camera parameters
	void SetProjection(float fFoV, float fAspect, float fZNear = 0.1f, float fZFar = 1000.0f);
	void SetUpVector(Vector &xUp);
	void SetLookAt(Vector &xFrom, Vector &xAt);

	// Make the camera active
	static void Set(Camera &rxCamera);

	static Camera &GetCurrentCamera(void);

	Matrix& GetViewMatrix()		{return m_xMtxView;}
	Matrix& GetProjMatrix()		{return m_xMtxProj;}
	Matrix& GetViewProjMatrix() {return m_xMtxViewProj;}		

	Vector& GetPosition()		{return m_xPosition;}

	Frustum&GetFrustum()		{return m_xFrustum;}

private:

	void  UpdateMatrices(void);

	Frustum m_xFrustum;

	Matrix m_xMtxView;
	Matrix m_xMtxProj;
	Matrix m_xMtxViewProj;
	
	Vector m_xPosition;		
	Vector m_xTarget;
	Vector m_xUp;

	float  m_fFOV;
	float  m_fAspect;
	float  m_fNearZ;
	float  m_fFarZ;
};



#endif  // __BASE_CAMERA_H__