/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Frustum Source (Independent)
* Author  : Dave Reed
*/

/*-Include-Once--------------------------------------------------------------*/

#include "Main.h"
#include <Base/Math.h>

/*-Classes-------------------------------------------------------------------*/

void NormalizePlane(Vector &plane)
{
	float  mag = sqrt(plane.m_fX * plane.m_fX + plane.m_fY * plane.m_fY + plane.m_fZ * plane.m_fZ);
	plane.m_fX = plane.m_fX / mag;
	plane.m_fY = plane.m_fY / mag;
	plane.m_fZ = plane.m_fZ / mag;
	plane.m_fW = plane.m_fW / mag;
}


void Frustum::FromMatrix(Matrix &mtx)
{
	// Left clipping plane
	m_aPlanes[0].m_fX = mtx[0][3] + mtx[0][0];  
	m_aPlanes[0].m_fY = mtx[1][3] + mtx[1][0]; 
	m_aPlanes[0].m_fZ = mtx[2][3] + mtx[2][0];
	m_aPlanes[0].m_fW = mtx[3][3] + mtx[3][0]; 
	
	// Right clipping plane
	m_aPlanes[1].m_fX = mtx[0][3] - mtx[0][0];
	m_aPlanes[1].m_fY = mtx[1][3] - mtx[1][0];
	m_aPlanes[1].m_fZ = mtx[2][3] - mtx[2][0];
	m_aPlanes[1].m_fW = mtx[3][3] - mtx[3][0];

	// Top clipping plane
	m_aPlanes[2].m_fX = mtx[0][3] - mtx[0][1]; 
	m_aPlanes[2].m_fY = mtx[1][3] - mtx[1][1]; 
	m_aPlanes[2].m_fZ = mtx[2][3] - mtx[2][1]; 
	m_aPlanes[2].m_fW = mtx[3][3] - mtx[3][1]; 

	// Bottom clipping plane
	m_aPlanes[3].m_fX = mtx[0][3] + mtx[0][1]; 
	m_aPlanes[3].m_fY = mtx[1][3] + mtx[1][1]; 
	m_aPlanes[3].m_fZ = mtx[2][3] + mtx[2][1]; 
	m_aPlanes[3].m_fW = mtx[3][3] + mtx[3][1]; 

	// Near clipping plane
	m_aPlanes[4].m_fX = mtx[0][2];
	m_aPlanes[4].m_fY = mtx[1][2];
	m_aPlanes[4].m_fZ = mtx[2][2];
	m_aPlanes[4].m_fW = mtx[3][2];
	
	// Far clipping plane
	m_aPlanes[5].m_fX = mtx[0][3] - mtx[0][2];
	m_aPlanes[5].m_fY = mtx[1][3] - mtx[1][2];
	m_aPlanes[5].m_fZ = mtx[2][3] - mtx[2][2];
	m_aPlanes[5].m_fW = mtx[3][3] - mtx[3][2];	

	for(int i=0; i<6; i++)
	{
		NormalizePlane(m_aPlanes[i]);
	}
}


bool Frustum::SphereInFrustum(Vector &vPos, float fRad)
{
	for(int i=0; i<6; i++)
	{
			if ((m_aPlanes[i].m_fX * vPos.m_fX +
				 m_aPlanes[i].m_fY * vPos.m_fY +
				 m_aPlanes[i].m_fZ * vPos.m_fZ +
				 m_aPlanes[i].m_fW) < -fRad) return false;
	
			//plane.a*pt.x + plane.b*pt.y + plane.c*pt.z + plane.d
	}

	return true;
}


bool Frustum::SphereInFrustum2D(Vector &vPos, float fRad)
{
	for(int i=0; i<4; i++)
	{
		if ((m_aPlanes[i].m_fX * vPos.m_fX +
			m_aPlanes[i].m_fY * vPos.m_fY +
			m_aPlanes[i].m_fZ * vPos.m_fZ +
			m_aPlanes[i].m_fW) < -fRad) return false;

		//plane.a*pt.x + plane.b*pt.y + plane.c*pt.z + plane.d
	}

	return true;
}


