/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Input Header (Independent)
* Author  : Dave Reed
*/

/*-Include-Once--------------------------------------------------------------*/

#if !defined(__BASE_INPUT_H__)
#define __BASE_INPUT_H__ /*!< \brief Include once define. */


/*-Classes-------------------------------------------------------------------*/

// Actions - Digital inputs, ABXY are assumed to be 
// in the Xbox positions

enum BASE_ACTION
{
	ACTION_LEFT,
	ACTION_RIGHT,
	ACTION_UP,
	ACTION_DOWN,
	ACTION_BUTTONA,		// Z on keyboard
	ACTION_BUTTONB,		// X on keyboard
	ACTION_BUTTONX,		// A on keyboard
	ACTION_BUTTONY,     // S on keyboard
	ACTION_PAUSE,		// Enter on keyboard (todo, move to Esc)
	ACTION_BUTTON1,
	ACTION_BUTTON2,
	ACTION_BUTTON3,
	ACTION_BUTTON4,
	ACTION_RESTART,
	NUM_ACTIONS
};


class Input
{

public:

	static void Initialize();
	static void Shutdown();
	static void Update();

	static bool ActionHeld(BASE_ACTION eAction);
	static bool ActionPressed(BASE_ACTION eAction);

	static float MouseX();
	static float MouseY();
	static int   MouseButtonsHeld()    {return s_iMouseB;}
	static int   MouseButtonsPressed() {return s_iMouseB & (~s_iPrevMouseB);}
	static bool  MouseValid()		   {return s_bMouseValid;}

private:

	static void Platform_Initialize();
	static void Platform_Shutdown();
	static void Platform_Update();

	static bool s_abActions[NUM_ACTIONS];
	static bool s_abPrevActions[NUM_ACTIONS];

	static int  s_iMouseX;
	static int  s_iMouseY;
	static int  s_iMouseB;
	static int  s_iPrevMouseB;
	static bool s_bMouseValid;

};


#endif // __BASE_INPUT_H_