/*! \file
 * Copyright (c) 2008
 * Project : Base
 * File    : Math Header (Independent)
 * Author  : Dave Reed
 */

/*-Include-Once--------------------------------------------------------------*/

#if !defined(__BASE_MATH_H__)
#define __BASE_MATH_H__ /*!< \brief Include once define. */

#include <math.h>

#include "Base/Vector.h"
#include "Base/Matrix.h"
#include "Base/Frustum.h"

/*-Classes-------------------------------------------------------------------*/

/*!
 * \brief The Math class - Misc maths functions, mostly wrappers around
 * C library functions for now
 */
class Math
{
public:

	static float Clamp(const float fValue, const float fMin, const float fMax);
	static float Lerp(const float fSrc, const float fDest, float fLerp);

	static float Sin( const float fRadians );
	static float Cos( const float fRadians );
	static float Tan( const float fRadians );

	static float ASin( const float fSin );
	static float ACos( const float fCos );
	static float ATan( const float fTan );

	static float DegToRad( const float fDegrees );
	static float RadToDeg( const float fRadians );

	static float Sqrt( const float fInput );
	static float FAbs( const float fInput );

	static float Pi( void );
};

#endif /* __BASE_MATH_H__ */
