
/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Main.cpp
* Author  : Dave Reed
*/

/*-Project-Includes----------------------------------------------------------*/

#include <Windows.h>
#include "Main.h"
#include "Game.h"
#include "Base/Render.h"
#include "Base/Shaders.h"
#include "Base/Blitter.h"
#include "Base/Input.h"
#include "Base/PostProcess.h"
#include "Platform_Render.h"

static bool s_bExit = false;


static int GetTickStep()
{
	LARGE_INTEGER counterTicks;
	LARGE_INTEGER counterFreq;
	double        tickCount;

	QueryPerformanceCounter(&counterTicks);
	QueryPerformanceFrequency(&counterFreq);

	static LONGLONG last = 0;	
	
	tickCount=(double)(counterTicks.QuadPart- last);
	tickCount/=((double)(counterFreq.QuadPart)*0.001f);

	last = counterTicks.QuadPart ;

	return (int)(tickCount);
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR szCmdLine, int iShowCmd)
{	

	Platform_Render::SetHInst(hInstance);

	// Base Init
	RenderConfig xConfig;
	Render::Initialize(xConfig);
	Shaders::Initialize();
	Texture::Initialize();
	Blitter2D::Initialize();
	Blitter3D::Initialize();
	PostProcess::Initialize();
	Input::Initialize();

	// Game Init
	Game::Initialize();	
	
	while (!s_bExit)
	{
		MSG Msg;
		while(PeekMessage(&Msg, NULL, 0, 0, PM_REMOVE) != 0)
		{
			/* Process message */
			TranslateMessage(&Msg);
			DispatchMessage(&Msg);

			/* Check message is a quit message */
			if(Msg.message == WM_QUIT)
			{
				/* Stop running */
				s_bExit = true;
			}
		}
		/* Handle exit requests from the Render system */
		s_bExit |= Render::IsExitRequested();

		if (!s_bExit)
		{
			if (!Platform_Render::IsRenderingPaused())
			{				
				Render::PresentFrame();

				// Todo - remove this, when the Render or Viewport class has a suitable Clear() function				
				Platform_Render::GetD3DDevice()->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER | D3DCLEAR_STENCIL,  0x80808080, 1.0f, 0 );
			}

			if(!Platform_Render::IsRenderingFocused())
			{
				/* Main window is not active, yield the CPU */
				Sleep(50);
			}

			// Update input
			Input::Update();
				
			// Calc timestep			
			int iStep = GetTickStep();
			float fTimeStep = (float(iStep))/1000.0f;
			if (fTimeStep<=0.0f) fTimeStep = 1.0f/250.0f;
			if (fTimeStep>=0.1f) fTimeStep = 1.0f/50.0f;			
			
			// Update+render
			Game::UpdateAndRender(fTimeStep);
		}		
	}

	// Game Shutdown
	Game::Shutdown();

	// Base Shutdown
	Input::Shutdown();
	PostProcess::Shutdown();
	Blitter3D::Shutdown();
	Blitter2D::Shutdown();
	Texture::Shutdown();
	Shaders::Shutdown();
	Render::Shutdown();
	
}