/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Platform_Shader Source (Windows)
* Author  : Dave Reed
*/

/*-Project-Includes----------------------------------------------------------*/

#include "Main.h"
#include "Platform_Input.h"
#include "Platform_Render.h"

#define DIRECTINPUT_VERSION 0x0800

#include <dinput.h>

/*-Classes-------------------------------------------------------------------*/


LPDIRECTINPUT8		  g_pDirectInput				= NULL; 
LPDIRECTINPUTDEVICE8  g_pDirectInputKeyboardDevice	= NULL; 
LPDIRECTINPUTDEVICE8  g_pDirectInputJoystickDevice	= NULL; 
LPDIRECTINPUTDEVICE8  g_pDirectInputMouseDevice		= NULL; 
int					  g_iMouseX;
int					  g_iMouseY;
bool				  g_bMouseValid;


BOOL CALLBACK EnumJoysticksCallback(const DIDEVICEINSTANCE* pdidInstance, VOID* pContext)
{
	HRESULT hr;

	// Obtain an interface to the enumerated joystick.
	hr = g_pDirectInput->CreateDevice(pdidInstance->guidInstance,  &g_pDirectInputJoystickDevice, NULL);
	
	if(FAILED(hr)) 
		return DIENUM_CONTINUE;
	else
		return DIENUM_STOP;
}


void Input::Platform_Initialize(void)
{	
	DirectInput8Create(Platform_Render::GetHInst(), DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&g_pDirectInput, NULL); 
	g_pDirectInput->CreateDevice(GUID_SysKeyboard, &g_pDirectInputKeyboardDevice, NULL); 
	g_pDirectInputKeyboardDevice->SetDataFormat(&c_dfDIKeyboard); 
	g_pDirectInputKeyboardDevice->SetCooperativeLevel(Platform_Render::GetHWnd(), DISCL_FOREGROUND | DISCL_NONEXCLUSIVE); 
	g_pDirectInputKeyboardDevice->Acquire();

	g_pDirectInput->EnumDevices(DI8DEVCLASS_GAMECTRL, EnumJoysticksCallback, NULL, DIEDFL_ATTACHEDONLY);

	if (g_pDirectInputJoystickDevice)
	{
		g_pDirectInputJoystickDevice->SetDataFormat(&c_dfDIJoystick2);
		g_pDirectInputJoystickDevice->SetCooperativeLevel(Platform_Render::GetHWnd(), DISCL_FOREGROUND | DISCL_EXCLUSIVE); 
	}

	g_pDirectInput->CreateDevice(GUID_SysMouse, &g_pDirectInputMouseDevice, NULL);
	g_pDirectInputMouseDevice->SetDataFormat(&c_dfDIMouse);
	g_pDirectInputMouseDevice->SetCooperativeLevel(Platform_Render::GetHWnd(), DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);


}



void Input::Platform_Shutdown(void)
{

	if (g_pDirectInputMouseDevice)
	{	
		g_pDirectInputMouseDevice->Release();
		g_pDirectInputMouseDevice = NULL;
	}

	if (g_pDirectInputKeyboardDevice)
	{	
		g_pDirectInputKeyboardDevice->Release();
		g_pDirectInputKeyboardDevice = NULL;
	}

	if (g_pDirectInputJoystickDevice)
	{	
		g_pDirectInputJoystickDevice->Release();
		g_pDirectInputJoystickDevice = NULL;
	}

	g_pDirectInput->Release();
	g_pDirectInput = NULL;
}



void Input::Platform_Update(void)
{
	// Read the keyboard
	#define KEYDOWN(key) ((aKeyState[key] & 0x80) != 0)

	char     aKeyState[256]; 
	HRESULT  hr; 
	hr = g_pDirectInputKeyboardDevice->GetDeviceState(sizeof(aKeyState),(LPVOID)&aKeyState); 

	if FAILED(hr) 
	{ 
		g_pDirectInputKeyboardDevice->Acquire();		
		return; 
	} 
	
	s_abActions[ACTION_LEFT]	= KEYDOWN(DIK_LEFT);
	s_abActions[ACTION_RIGHT]	= KEYDOWN(DIK_RIGHT);
	s_abActions[ACTION_UP]		= KEYDOWN(DIK_UP);
	s_abActions[ACTION_DOWN]	= KEYDOWN(DIK_DOWN);
	s_abActions[ACTION_BUTTONA]	= KEYDOWN(DIK_Z);
	s_abActions[ACTION_BUTTONB] = KEYDOWN(DIK_X);
	s_abActions[ACTION_BUTTONX]	= KEYDOWN(DIK_A);
	s_abActions[ACTION_RESTART]	= KEYDOWN(DIK_SPACE);
	s_abActions[ACTION_BUTTONY]	= KEYDOWN(DIK_S);
	s_abActions[ACTION_PAUSE]	= KEYDOWN(DIK_RETURN);
	s_abActions[ACTION_BUTTON1]	= KEYDOWN(DIK_1);
	s_abActions[ACTION_BUTTON2] = KEYDOWN(DIK_2);
	s_abActions[ACTION_BUTTON3]	= KEYDOWN(DIK_3);
	s_abActions[ACTION_BUTTON4]	= KEYDOWN(DIK_4);

	// Read the joystick, if we have one
	if (g_pDirectInputJoystickDevice)
	{
		hr = g_pDirectInputJoystickDevice->Poll();
		if (FAILED(hr))
		{
			g_pDirectInputJoystickDevice->Acquire();
			return;
		}

		DIJOYSTATE2 js;
		hr = g_pDirectInputJoystickDevice->GetDeviceState(sizeof(DIJOYSTATE2), &js);
		if (FAILED(hr))
		{
			return;
		}

		if (js.rgdwPOV[0]!=0xffffffff)
		{
			if (js.rgdwPOV[0] <= 4500  || js.rgdwPOV[0] >= 31500) s_abActions[ACTION_UP]    = true;
			if (js.rgdwPOV[0] >= 4500  && js.rgdwPOV[0] <= 13500) s_abActions[ACTION_RIGHT] = true;
			if (js.rgdwPOV[0] >= 13500 && js.rgdwPOV[0] <= 22500) s_abActions[ACTION_DOWN]  = true;
			if (js.rgdwPOV[0] >= 22500 && js.rgdwPOV[0] <= 31500) s_abActions[ACTION_LEFT]  = true;
		}

		s_abActions[ACTION_BUTTONA]	|= ((js.rgbButtons[0]&0x80) !=0);
		s_abActions[ACTION_BUTTONB] |= ((js.rgbButtons[1]&0x80) !=0);
		s_abActions[ACTION_BUTTONX]	|= ((js.rgbButtons[2]&0x80) !=0);
		s_abActions[ACTION_BUTTONY]	|= ((js.rgbButtons[3]&0x80) !=0);
		s_abActions[ACTION_PAUSE]	|= ((js.rgbButtons[7]&0x80) !=0);

	}



	DIMOUSESTATE diMouseState;

	if(g_pDirectInputMouseDevice->GetDeviceState(sizeof(diMouseState), &diMouseState) == DI_OK)
	{
		s_iPrevMouseB = s_iMouseB;
		s_iMouseB  = diMouseState.rgbButtons[0]?1:0;
		s_iMouseB += diMouseState.rgbButtons[1]?2:0;
		s_iMouseX     = g_iMouseX;
		s_iMouseY     = g_iMouseY;
		s_bMouseValid = g_bMouseValid;

		if (s_iMouseX<0) s_iMouseX=0;
		if (s_iMouseY<0) s_iMouseY=0;
		if (s_iMouseX>Render::GetScreenWidth())  s_iMouseX=Render::GetScreenWidth();
		if (s_iMouseY>Render::GetScreenHeight()) s_iMouseY=Render::GetScreenHeight();
	}
	else
	{
		g_pDirectInputMouseDevice->Acquire();
	}
}

void Platform_Input_SetMousePos(int x, int y)
{
	if (x==-1 && y==-1)
	{
		g_bMouseValid = false;
		g_iMouseX = -1000;
		g_iMouseY = -1000;
	}
	else
	{
		g_bMouseValid = true;
		g_iMouseX = x;
		g_iMouseY = y;
	}	
}
