/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Platform_Render.cpp
* Author  : Dave Reed
*/

/*-Project-Includes----------------------------------------------------------*/

#include "Main.h"

#include <winuser.h>
#include <windowsx.h>

#include "Platform_Render.h"
#include "Platform_Shaders.h"
#include "Platform_Input.h"

/*-Class-Static-Variables----------------------------------------------------*/

HWND					Platform_Render::s_hWnd;
HINSTANCE				Platform_Render::s_hInst;
D3DPRESENT_PARAMETERS	Platform_Render::s_xD3Dpp;
RECT					Platform_Render::s_xRect;
bool					Platform_Render::s_bPaused;
bool					Platform_Render::s_bNotFocused;
bool					Platform_Render::s_bMaximized=false;
bool					Platform_Render::s_bExitRequested;
bool					Platform_Render::s_bDoingSizeMove;
int						Platform_Render::s_iScreenWidth = 0;
int						Platform_Render::s_iScreenHeight = 0;
IDirect3D9*				Platform_Render::s_pxD3D = NULL;
IDirect3DDevice9*		Platform_Render::s_pxD3DDevice = NULL;
IDirect3DSurface9*		Platform_Render::s_pxD3DBackBufferSurface = NULL;
IDirect3DSurface9*		Platform_Render::s_pxD3DDepthStencilSurface = NULL;
Matrix					Platform_Render::s_xMtxViewProj;
Matrix					Platform_Render::s_xMtxView;
Matrix					Platform_Render::s_xMtxWorld;		
Viewport				Platform_Render::s_xDefaultViewport;
Viewport				Platform_Render::s_xCurrentViewport;

int						Platform_Render::s_iDesktopWidth = 0;
int						Platform_Render::s_iDesktopHeight = 0;

/*-Class-Public-Functions----------------------------------------------------*/

bool Platform_Render::Initialize(void)
{
	RenderConfig& xConfig = Render::GetCurrentConfig();

	InitialiseWindow();
	
	// Create D3D objects
	s_pxD3D = Direct3DCreate9(D3D_SDK_VERSION);

	// Set up screen format
	ZeroMemory( &s_xD3Dpp, sizeof(s_xD3Dpp) );

	s_xD3Dpp.Windowed				= xConfig.m_bWindowed;
	if (xConfig.m_bWindowed)
	{
		int iWidth, iHeight;
		GetWindowDimensions(&iWidth, &iHeight);
		s_xD3Dpp.BackBufferWidth  = s_iScreenWidth  = iWidth;
		s_xD3Dpp.BackBufferHeight = s_iScreenHeight = iHeight;
	}
	else
	{
		s_xD3Dpp.BackBufferWidth  = s_iScreenWidth = xConfig.m_iFullScreenWidth;
		s_xD3Dpp.BackBufferHeight = s_iScreenHeight= xConfig.m_iFullScreenHeight;
	}

	s_xD3Dpp.SwapEffect				= D3DSWAPEFFECT_DISCARD;
	s_xD3Dpp.BackBufferFormat		= D3DFMT_UNKNOWN;
	s_xD3Dpp.BackBufferFormat		= D3DFMT_X8R8G8B8;
	s_xD3Dpp.MultiSampleType		= D3DMULTISAMPLE_NONE;
	s_xD3Dpp.EnableAutoDepthStencil = true;
	s_xD3Dpp.AutoDepthStencilFormat	= D3DFMT_D24S8;
	s_xD3Dpp.PresentationInterval   = D3DPRESENT_INTERVAL_ONE;

	if (xConfig.m_iMultisample==2)  s_xD3Dpp.MultiSampleType = D3DMULTISAMPLE_2_SAMPLES;
	if (xConfig.m_iMultisample==4)  s_xD3Dpp.MultiSampleType = D3DMULTISAMPLE_4_SAMPLES;

	if (xConfig.m_iMultisample>0)
	{
		DWORD iMultiSampleQualityLevels = 0;
		HRESULT hr = s_pxD3D->CheckDeviceMultiSampleType(0,
			D3DDEVTYPE_HAL,
			s_xD3Dpp.BackBufferFormat,
			s_xD3Dpp.Windowed,
			s_xD3Dpp.MultiSampleType,
			&iMultiSampleQualityLevels
			);

		if (hr == D3D_OK)
		{
			s_xD3Dpp.MultiSampleQuality = iMultiSampleQualityLevels-1;
		}
		else
		{
			return false;
		}
	}

	// Create the device
	s_pxD3D->CreateDevice(	D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, s_hWnd,
		D3DCREATE_HARDWARE_VERTEXPROCESSING | D3DCREATE_PUREDEVICE ,
		&s_xD3Dpp,
		&s_pxD3DDevice);

	// Clear the back buffer
	s_pxD3DDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER | D3DCLEAR_STENCIL, Render::GetCurrentConfig().m_uClearARGB , 1.0f, 0 );

	// Get the back buffer
	GetD3DDevice()->GetBackBuffer(0,0,D3DBACKBUFFER_TYPE_MONO,&s_pxD3DBackBufferSurface);
	GetD3DDevice()->GetDepthStencilSurface(&s_pxD3DDepthStencilSurface);

	// Hide the cursor in fullscreen mode
	ShowCursor(FALSE);

	// Begin frame
	GetD3DDevice()->BeginScene();
	s_xDefaultViewport.Set(0,0, Render::GetScreenWidth(), Render::GetScreenHeight());
	SetViewport(NULL);

	ShowWindow(s_hWnd,SW_SHOWNORMAL);
	SetCursor(LoadCursor(s_hInst,IDC_ARROW));
	ShowWindow(s_hWnd,SW_SHOWNORMAL);
	SetCursor(LoadCursor(s_hInst,IDC_ARROW));		

	RECT rect;
	GetClientRect(GetDesktopWindow(), &rect);
	s_iDesktopWidth  = rect.right;
	s_iDesktopHeight = rect.bottom;

	return true;
}

void Platform_Render::GetWindowDimensions(int *piWidth, int *piHeight)
{
	RECT xRect;
	GetClientRect(s_hWnd, &xRect);

	int iWidth  = xRect.right-xRect.left;
	int iHeight = xRect.bottom-xRect.top;

	if (iWidth<1)  iWidth=1;
	if (iHeight<1) iHeight=1;

	*piWidth  = iWidth;
	*piHeight = iHeight;
}

void Platform_Render::Shutdown(void)
{
	D3D_SAFE_RELEASE(s_pxD3DDepthStencilSurface);
	D3D_SAFE_RELEASE(s_pxD3DBackBufferSurface);
	D3D_SAFE_RELEASE(s_pxD3DDevice);
	D3D_SAFE_RELEASE(s_pxD3D);
}

void Platform_Render::InitialiseWindow()
{
	RenderConfig& xConfig = Render::GetCurrentConfig();

	// Register a window class
	// ---------------------------------------------------
	WNDCLASSEX xWC = {sizeof(WNDCLASSEX),
		CS_CLASSDC,
		Platform_Render::WindowProc,
		0L,0L,
		GetModuleHandle(NULL),
		NULL,NULL,NULL,NULL,
		xConfig.m_szProcessName,
		0}; //LoadIcon(s_hInst,MAKEINTRESOURCE(IDI_ICON1))};

	RegisterClassEx(&xWC);


	// Create the window
	// ---------------------------------------------------
	s_xRect.left   = s_xRect.top = 0;
	s_xRect.right  = xConfig.m_iWindowedWidth;
	s_xRect.bottom = xConfig.m_iWindowedHeight;

	s_xRect.right  += GetSystemMetrics(SM_CXSIZEFRAME )*2;
	s_xRect.bottom += GetSystemMetrics(SM_CYSIZEFRAME )*2;
	s_xRect.bottom += GetSystemMetrics(SM_CYCAPTION );

	s_hWnd = CreateWindow(  xConfig.m_szProcessName,
		xConfig.m_szWindowTitle,
		/*WS_OVERLAPPED|WS_MINIMIZEBOX|WS_SYSMENU|WS_THICKFRAME,*/ WS_OVERLAPPEDWINDOW,  // - odd problem with the maximised state when restoring from fullscreen :(
		32,32,
		s_xRect.right, s_xRect.bottom,
		GetDesktopWindow(),
		NULL, xWC.hInstance, NULL );
}

LRESULT WINAPI Platform_Render::WindowProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch(uMsg)
	{
		// If doing a resize/move, we've got to respond to paint
		// messages. Ideally we'd render a real frame, but a
		// black screen is better than a smear of window frame..
		// -------------------------------------------------------
	case WM_PAINT:
		if (s_bDoingSizeMove)
		{
			PresentFrame();
		}
		break;

		// Handle alt-enter to toggle full-screen mode
		// -------------------------------------------------------
	case WM_SYSKEYDOWN:
		{
			// Alt-enter
			bool  bAltDown = ( (lParam & (1 << 29)) != 0 );
			if (bAltDown && wParam == 0x0D)
			{
				ToggleFullScreen();
			}
		}
		break;

		// Handle Esc to quit, but not if in tools mode
		// -------------------------------------------------------
	case WM_KEYDOWN:
		if (wParam == 0x1B)
		{
			s_bExitRequested = true;
		}			
		break;


		// Quit if D3D window is closed
		// -------------------------------------------------------
	case WM_DESTROY:
		PostQuitMessage(0);
		s_bExitRequested = true;
		break;


		// Handle resizing
		// -------------------------------------------------------
	case WM_SIZE:

		// Update stored window rect
		if (s_xD3Dpp.Windowed)
		{
			GetWindowRect(hWnd, &s_xRect);			
		}

		if (s_xD3Dpp.Windowed)
		{
			s_bMaximized = false;
		}

		if( SIZE_MAXIMIZED == wParam )
		{
			if (s_xD3Dpp.Windowed) 
			{
				s_bMaximized = true;
			}

			s_bPaused = false;
			Reset();
		}
		else if( SIZE_RESTORED == wParam )
		{
			if (!s_bDoingSizeMove)
			{
				s_bPaused = false;
				Reset();
			}
		}
		break;

	case WM_ENTERSIZEMOVE:
		s_bDoingSizeMove = true;
		break;

	case WM_EXITSIZEMOVE:
		s_bDoingSizeMove = false;
		
		if (s_xD3Dpp.Windowed)
		{
			GetWindowRect(hWnd, &s_xRect);			
		}

		Reset();
		break;

	case WM_MOUSELEAVE:
		Platform_Input_SetMousePos( -1, -1);
		return 0;
		break;

	case WM_MOUSEMOVE:
		Platform_Input_SetMousePos( GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));		
		SetTimer(s_hWnd,100,15,NULL);
		break;

	case WM_TIMER:
		{		
			RECT rc;
			POINT pt;
			GetWindowRect(s_hWnd,&rc);
			GetCursorPos(&pt);			
			if(!PtInRect(&rc,pt))
			{
				PostMessage(s_hWnd,WM_MOUSELEAVE,0,0L);
				KillTimer(s_hWnd,100);								
				return 0;
			}

		}
		break;

		// Prevent moving/sizing in full screen mode
		// -------------------------------------------------------
	case WM_SYSCOMMAND:
		switch( (wParam & 0xFFF0) )
		{
		case SC_MOVE:
		case SC_SIZE:
		case SC_MAXIMIZE:
		case SC_KEYMENU:
			if( !Render::GetCurrentConfig().m_bWindowed )
			{
				return 0;
			}
			break;
		}
		break;

		// Handle pause/reset on minimize/alt-tab
		// -------------------------------------------------------
	case WM_ACTIVATE:
		if (LOWORD(wParam)==WA_ACTIVE ||  LOWORD(wParam)==WA_CLICKACTIVE)
		{
			s_bPaused = false;
			s_bNotFocused = false;

			if (!s_xD3Dpp.Windowed)
			{
				Reset();
				ShowWindow(s_hWnd, SW_SHOW);
				ShowCursor(FALSE);
			}
		}

		if (LOWORD(wParam)==WA_INACTIVE)
		{
			if (!s_xD3Dpp.Windowed)
			{
				s_bPaused = true;
			}
			else
			{
				if (HIWORD(wParam))
				{
					s_bPaused = true;
				}
				else
				{
					s_bNotFocused = true;
				}
			}

			ShowCursor(FALSE);
		}
		break;
	}

	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}

void Platform_Render::ShowCursor(bool bShow)
{
	static bool s_bState=true;
	if (bShow!=s_bState) ::ShowCursor(bShow);
	s_bState=bShow;
}

void Platform_Render::PresentFrame(void)
{
	if (s_pxD3DDevice)
	{
		Render::Flush();

		s_pxD3DDevice->EndScene();
		s_pxD3DDevice->Present( NULL, NULL, NULL, NULL );
		s_pxD3DDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER | D3DCLEAR_STENCIL,  Render::GetCurrentConfig().m_uClearARGB , 1.0f, 0 );
		s_pxD3DDevice->BeginScene();

		s_xDefaultViewport.Set(0,0, Render::GetScreenWidth(), Render::GetScreenHeight());
		SetViewport(NULL);
	}
}

void Platform_Render::ToggleFullScreen(void)
{
	RenderConfig& xConfig = Render::GetCurrentConfig();

	s_xD3Dpp.Windowed=!s_xD3Dpp.Windowed;
/*
	int iWidth  = s_xRect.right - s_xRect.left;
	int iHeight = s_xRect.bottom - s_xRect.top;
	
	iWidth  -= GetSystemMetrics(SM_CXSIZEFRAME )*2;
	iHeight -= GetSystemMetrics(SM_CYSIZEFRAME )*2;
	iHeight -= GetSystemMetrics(SM_CYCAPTION );

	if (s_xD3Dpp.Windowed)
	{
		s_xD3Dpp.BackBufferWidth  = s_iScreenWidth  = iWidth;
		s_xD3Dpp.BackBufferHeight = s_iScreenHeight = iHeight;
	}
	else
	{
		s_xD3Dpp.BackBufferWidth	= s_iScreenWidth  = xConfig.m_iFullScreenWidth;
		s_xD3Dpp.BackBufferHeight	= s_iScreenHeight = xConfig.m_iFullScreenHeight;
	}
*/
	Reset();

		
	if (s_xD3Dpp.Windowed)
	{
		bool bMax = s_bMaximized;
		SetWindowPos(s_hWnd,HWND_TOP,s_xRect.left,s_xRect.top,s_xRect.right-s_xRect.left,s_xRect.bottom-s_xRect.top,0);

		if (bMax)
		{		
			ShowWindow(s_hWnd, SW_MAXIMIZE);
		}
		else
		{		
			ShowWindow(s_hWnd, SW_SHOWNORMAL);
		}

		SetFocus(s_hWnd);
		ShowCursor(FALSE);
	}
	else
	{
		ShowCursor(FALSE);
		ShowWindow(s_hWnd, SW_SHOW);
	}		
}

void Platform_Render::Reset(void)
{
	if (s_pxD3DDevice && !s_bPaused)
	{
		RenderConfig& xConfig = Render::GetCurrentConfig();

		if (s_xD3Dpp.Windowed)
		{
			int iWidth  = s_xRect.right - s_xRect.left;
			int iHeight = s_xRect.bottom - s_xRect.top;			

			iWidth  -= GetSystemMetrics(SM_CXSIZEFRAME )*2;
			iHeight -= GetSystemMetrics(SM_CYSIZEFRAME )*2;
			iHeight -= GetSystemMetrics(SM_CYCAPTION );

			if (iWidth!=s_iScreenWidth || iHeight!=s_iScreenHeight)
			{
				s_iScreenWidth  = iWidth;
				s_iScreenHeight = iHeight;
			}
		}
		else
		{
			s_iScreenWidth  = xConfig.m_iFullScreenWidth;
			s_iScreenHeight = xConfig.m_iFullScreenHeight;			
		}

		if (s_iScreenWidth<=0)  s_iScreenWidth  = 1;
		if (s_iScreenHeight<=0) s_iScreenHeight = 1;

		s_xD3Dpp.BackBufferWidth  = s_iScreenWidth;
		s_xD3Dpp.BackBufferHeight = s_iScreenHeight;

		

		GetD3DDevice()->EndScene();

		//			*** TODO: DESTROY RENDER TARGETS HERE ***

		D3D_SAFE_RELEASE(s_pxD3DBackBufferSurface);
		D3D_SAFE_RELEASE(s_pxD3DDepthStencilSurface);

		if (xConfig.m_iMultisample>0)
		{
			DWORD iMultiSampleQualityLevels = 0;

			s_pxD3D->CheckDeviceMultiSampleType(0,
				D3DDEVTYPE_HAL,
				s_xD3Dpp.BackBufferFormat,
				s_xD3Dpp.Windowed,
				s_xD3Dpp.MultiSampleType,
				&iMultiSampleQualityLevels
				);

			s_xD3Dpp.MultiSampleQuality     = iMultiSampleQualityLevels-1;
		}

		Platform_Shaders::PreReset();
		Texture::PreReset();

		s_pxD3DDevice->Reset(&s_xD3Dpp);
		s_pxD3DDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER | D3DCLEAR_STENCIL, Render::GetCurrentConfig().m_uClearARGB , 1.0f, 0 );

		Texture::PostReset();
		Platform_Shaders::PostReset();

		GetD3DDevice()->GetBackBuffer(0,0,D3DBACKBUFFER_TYPE_MONO,&s_pxD3DBackBufferSurface);
		GetD3DDevice()->GetDepthStencilSurface(&s_pxD3DDepthStencilSurface);

		//			*** TODO: RECREATE RENDER TARGETS HERE ***

		GetD3DDevice()->BeginScene();
	}
}






void Platform_Render::SetWorldMatrix( Matrix &xMtxWorld )
{
	s_xMtxWorld = xMtxWorld;
	Matrix xWorldViewProj = s_xMtxWorld * s_xMtxViewProj;	
	Platform_Shaders::SetSharedParamMatrix44(SHADER_PARAM_WORLDVIEWPROJ, xWorldViewProj);			
}



void Platform_Render::SetCamera(Camera &rxCamera)
{
	Render::Flush();

	// Store camera matrices
	s_xMtxViewProj = rxCamera.GetViewProjMatrix();
	s_xMtxView	   = rxCamera.GetViewMatrix();

	// Reset world matrix
	Matrix xIdentity;
	xIdentity.SetIdentity();
	SetWorldMatrix(xIdentity);
}


void Platform_Render::SetViewport(Viewport *pxViewport)
{
	Viewport *pxVP = pxViewport;
	if (!pxVP) pxVP = &s_xDefaultViewport;

	Render::Flush();

	D3DVIEWPORT9 xViewport; 

	xViewport.X		 = pxVP->GetOriginX();
	xViewport.Y		 = pxVP->GetOriginY();
	xViewport.Width  = pxVP->GetWidth();
	xViewport.Height = pxVP->GetHeight();
	xViewport.MinZ   = 0.0f;
	xViewport.MaxZ   = 1.0f;

	if (pxVP->GetWidth()<=0)  xViewport.Width  = 1;
	if (pxVP->GetHeight()<=0) xViewport.Height = 1;

	GetD3DDevice()->SetViewport(&xViewport);

	s_xCurrentViewport = *pxVP;

	// Set up pixel->viewport scaling
	Render::Reset2DScaling();
	Render::Reset2DTransform();

	
	
}


void Platform_Render::ClearViewport(uint32 uClearARGB, bool bClearColour, bool bClearDepth, bool bClearStencil)
{
	uint32 uFlags = 0;

	if (bClearColour)  uFlags |= D3DCLEAR_TARGET;
	if (bClearDepth)   uFlags |= D3DCLEAR_ZBUFFER;
	if (bClearStencil) uFlags |= D3DCLEAR_STENCIL;

	GetD3DDevice()->Clear( 0, NULL, uFlags, uClearARGB, 1.0f, 0 );
}



float Platform_Render::GetAspectRatio(void)
{
	if (s_xD3Dpp.Windowed)
	{
		// Windowed - we assume pixels are square
		return (float)Render::GetScreenHeight() / (float)Render::GetScreenWidth();
	}
	else
	{
		// Fullscreen - assume *desktop* pixels are square
		return ((float)s_iDesktopHeight / (float)s_iDesktopWidth);
	}
}