/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Platform_Render Header (Windows)
* Author  : Dave Reed
*/

/*-Include-Once--------------------------------------------------------------*/

#if !defined(__BASE_PLATFORM_RENDER_H__)
#define __BASE_PLATFORM_RENDER_H__ /*!< \brief Include once define. */

/*-Project-Includes----------------------------------------------------------*/

#include "Base/Render.h"

/*-Defines-------------------------------------------------------------------*/

#define D3D_SAFE_RELEASE(x) { if(x) { x->Release(); x = NULL; } }

/*-Classes-------------------------------------------------------------------*/

class Platform_Render
{
public:

	static bool						Initialize(void);
	static void						Shutdown(void);
	static void						Reset(void);
	static void						PresentFrame(void);

	static void						ToggleFullScreen(void);
	static bool						IsExitRequested(void)    {return s_bExitRequested;}
	static IDirect3DDevice9*		GetD3DDevice()		     {return s_pxD3DDevice;}

	static int						GetScreenWidth(void)     {return s_iScreenWidth;}
	static int						GetScreenHeight(void)    {return s_iScreenHeight;}

	static int						GetViewportWidth(void)	 {return s_xCurrentViewport.GetWidth();}		
	static int						GetViewportHeight(void)  {return s_xCurrentViewport.GetHeight();}		

	static bool						IsRenderingPaused(void)	 {return s_bPaused;}
	static bool						IsRenderingFocused(void) {return !s_bNotFocused;}

	static void						SetCamera(Camera &xCamera);
	static void						SetWorldMatrix(Matrix &xMtxWorld);

	static void						UpdateStandardShaderParameters(class ShaderParameterSet *pxSharedParams);

	static void						SetViewport(Viewport *pxViewport);	
	static void						ClearViewport(uint32 uClearARGB, bool bClearColour, bool bClearDepth, bool bClearStencil);

	static float					GetAspectRatio(void);

	static HWND						GetHWnd(void)  {return s_hWnd;}
	static HINSTANCE				GetHInst(void) {return s_hInst;}
	static void						SetHInst(HINSTANCE hInst) {s_hInst = hInst;}	
protected:

	static void						InitialiseWindow();
	static void						GetWindowDimensions(int *piWidth, int *piHeight);
	static LRESULT WINAPI			WindowProc(HWND hWnd,UINT msg,WPARAM wParam,LPARAM lParam);
	static void						ShowCursor(bool bShow);

	static HWND						s_hWnd;
	static HINSTANCE				s_hInst;
	static D3DPRESENT_PARAMETERS	s_xD3Dpp; 
	static RECT						s_xRect;
	static bool						s_bPaused;
	static bool						s_bMaximized;
	static bool						s_bOwnWindow;
	static bool						s_bNotFocused;
	static bool						s_bDoingSizeMove;
	static bool						s_bExitRequested;
	static int						s_iScreenWidth;
	static int						s_iScreenHeight;
	static IDirect3D9			   *s_pxD3D;
	static IDirect3DDevice9		   *s_pxD3DDevice;
	static IDirect3DSurface9	   *s_pxD3DBackBufferSurface;
	static IDirect3DSurface9	   *s_pxD3DDepthStencilSurface;

	static int						s_iDesktopWidth;
	static int						s_iDesktopHeight;

	// Current view matrices
	static Matrix					s_xMtxViewProj;
	static Matrix					s_xMtxView;
	static Matrix					s_xMtxWorld;		

	static Viewport					s_xDefaultViewport;
	static Viewport				    s_xCurrentViewport;

};

#endif // __BASE_PLATFORM_RENDER_H__
