
/*! \file
* Copyright (c) 2008
* Project : Base
* File    : RenderStates Source (PC)
* Author  : Dave Reed
*/

/*-Project-Includes----------------------------------------------------------*/

#include "Main.h"
#include "Base/Render.h"
#include "Platform_Render.h"


// Depth Buffer states
// ------------------------------------------------------------------------

void RenderStates::Platform_SetZBufferMode( RS_ZBUFFER_MODE eMode )
{
	switch(eMode)
	{	
	case RS_ZBUFFER_OFF:
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_ZFUNC, D3DCMP_ALWAYS);
		break;

	case RS_ZBUFFER_LESSEQUAL:
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_ZFUNC, D3DCMP_LESSEQUAL);
		break;

	case RS_ZBUFFER_GREATEREQUAL:
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_ZFUNC, D3DCMP_GREATEREQUAL);
		break;
	}			
}

void RenderStates::Platform_SetZWriteEnable( bool bEnable )
{
	Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_ZWRITEENABLE, bEnable);
}

// Alpha Blending states
// ------------------------------------------------------------------------

void RenderStates::Platform_SetAlphaTest( bool bEnable, int iThreshold )
{
	Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_ALPHATESTENABLE, bEnable);
	Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_ALPHAREF, iThreshold);
}

void RenderStates::Platform_SetBlendMode( RS_BLEND_MODE eMode )
{
	switch (eMode)
	{
	case RS_BLEND_NONE:
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_ALPHABLENDENABLE, false);
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
		break;

	case RS_BLEND_NORMAL:
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
		break;

	case RS_BLEND_ADD:
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		break;

	case RS_BLEND_MULTIPLY:
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_DESTCOLOR);
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
		break;
	}
}

// Cullng states
// ------------------------------------------------------------------------

void RenderStates::Platform_SetCullMode( RS_CULL_MODE eMode )
{
	switch (eMode)
	{
	case RS_CULL_NONE:
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
		break;

	case RS_CULL_CW:
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);
		break;

	case RS_CULL_CCW:
		Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
		break;
	}
}

// Misc states
void RenderStates::Platform_SetColorWriteEnable( bool bRed, bool bGreen, bool bBlue, bool bAlpha )
{
	uint32 uFlags = 0;

	if (bRed)	uFlags|= D3DCOLORWRITEENABLE_RED;
	if (bGreen) uFlags|= D3DCOLORWRITEENABLE_GREEN;
	if (bBlue)	uFlags|= D3DCOLORWRITEENABLE_BLUE;
	if (bAlpha) uFlags|= D3DCOLORWRITEENABLE_ALPHA;

	Platform_Render::GetD3DDevice()->SetRenderState(D3DRS_COLORWRITEENABLE, uFlags);
}
