/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Platform_Shader Source (Windows)
* Author  : Dave Reed
*/

/*-Project-Includes----------------------------------------------------------*/

#include "Main.h"
#include "Base/Render.h"
#include "Platform_Shaders.h"

/*-Defines-------------------------------------------------------------------*/

ID3DXEffectPool				   *Platform_Shaders::s_pShaderPool = NULL;
D3DShader						Platform_Shaders::s_aShaders[NUM_SHADERS];
D3DXHANDLE						Platform_Shaders::s_aShaderParameters[NUM_SHARED_SHADER_PARAMS];
int								Platform_Shaders::s_iCurrentShader = -1;
IDirect3DVertexDeclaration9	   *Platform_Shaders::s_pCurrentVertexDecl = 0;

D3DVERTEXELEMENT9 dwVertexDeclBlitter2D[] = 
{
	{0,  0,  D3DDECLTYPE_FLOAT2,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
	{0,  8,  D3DDECLTYPE_FLOAT2,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
	{0, 16,  D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    0 },
	D3DDECL_END()
};

D3DVERTEXELEMENT9 dwVertexDeclBlitter3D[] = 
{
	{0,  0,  D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
	{0, 12,  D3DDECLTYPE_FLOAT2,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
	{0, 20,  D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    0 },
	D3DDECL_END()
};

D3DVERTEXELEMENT9 dwVertexDeclBlitter3DLit[] = 
{
	{0,  0,  D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
	{0, 12,  D3DDECLTYPE_FLOAT2,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
	{0, 20,  D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    0 },
	{0, 24,  D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },
	D3DDECL_END()
};



ShaderListEntry  g_aShaderList[NUM_SHADERS] = 
{
	{ "Base_Blitter2D.fxo",			 dwVertexDeclBlitter2D },
	{ "Base_Blitter3D.fxo",			 dwVertexDeclBlitter3D },
	{ "Base_Bloom.fxo",				 dwVertexDeclBlitter2D },
	{ "Base_Blur.fxo",				 dwVertexDeclBlitter2D },
	{ "LD48_Blitter3D_Lit.fxo",		 dwVertexDeclBlitter3DLit }
};


char *g_aszParamNames[NUM_SHARED_SHADER_PARAMS] =
{
	"g_f4x4WorldViewProj",
	"g_f4x42DXform",
	"g_f4PixelsToViewport",
	"g_f4EffectControl",
	"g_fLightingAmount"
};


void Platform_Shaders::Initialize(void)
{
	// Create an effect pool
	D3DXCreateEffectPool(&s_pShaderPool);

	// Load the shaders
	for(int i=0; i<NUM_SHADERS; i++)
	{
		IDirect3DDevice9 *pxD3DDevice = Platform_Render::GetD3DDevice();
		D3DShader *pShader = &s_aShaders[i];		
		LPD3DXBUFFER pErrors = NULL;

		void *pCompiledShader = NULL;
		int   iCompiledShaderSize;

		// Load the shader
		char szFileName[80];
		sprintf(szFileName, "data\\base\\shaders\\%s", g_aShaderList[i].szShaderFile);
		FILE *f = fopen(szFileName, "rb");

		if (f)
		{
			fseek(f, 0, SEEK_END);
			iCompiledShaderSize = ftell(f);
			fseek(f, 0, SEEK_SET);
			pCompiledShader = new uint8[iCompiledShaderSize];
			fread(pCompiledShader, 1, iCompiledShaderSize, f);
			
			HRESULT hr = D3DXCreateEffect(pxD3DDevice, pCompiledShader, iCompiledShaderSize, NULL, NULL, 0, s_pShaderPool, &pShader->m_pD3DXEffect, &pErrors);

			delete [] pCompiledShader;

			hr = pxD3DDevice->CreateVertexDeclaration(g_aShaderList[i].pVertexDecl, &pShader->m_pD3DVertexDecl);
		}
				
		// Get the texture parameters
		if (pShader->m_pD3DXEffect)
		{
			char tmp[20];			
			for(int i=0; i<MAX_TEXTURE_STAGES; i++)
			{
				sprintf(tmp, "g_Texture%d", i);
				pShader->m_aTextureHandles[i] = pShader->m_pD3DXEffect->GetParameterByName(NULL, tmp);				
			}
		}
	}

	// Get the shared shader parameters
	GetSharedShaderParams();
}



void Platform_Shaders::Shutdown(void)
{
	// Release shader params
	ReleaseSharedShaderParams();

	// Release shaders
	for(int i=0; i<NUM_SHADERS; i++)
	{
		if (s_aShaders[i].m_pD3DXEffect)
		{
			s_aShaders[i].m_pD3DXEffect->Release();
			s_aShaders[i].m_pD3DXEffect = NULL;
		}

		if (s_aShaders[i].m_pD3DVertexDecl)
		{
			s_aShaders[i].m_pD3DVertexDecl->Release();
			s_aShaders[i].m_pD3DVertexDecl = NULL;
		}
	}

	// Release shader pool
	s_pShaderPool->Release();
	s_pShaderPool = NULL;
}


void Platform_Shaders::PreReset(void)
{
	// Clear any texture params from shaders
	for(int i=0; i<NUM_SHADERS; i++)
	{
		D3DShader *pShader = &s_aShaders[i];		

		if (pShader->m_pD3DXEffect)
		{
			for(int j=0; j<MAX_TEXTURE_STAGES; j++)
			{
				if (pShader->m_aTextureHandles[j]!=NULL)
				{
					pShader->m_pD3DXEffect->SetTexture(pShader->m_aTextureHandles[j], NULL);
					pShader->m_pD3DXEffect->CommitChanges();
				}
			}
		}
	}

	// Release shared params
	ReleaseSharedShaderParams();

	s_pCurrentVertexDecl = NULL;
}



void Platform_Shaders::PostReset(void)
{
	// Release shader params
	GetSharedShaderParams();	
}


void Platform_Shaders::GetSharedShaderParams(void)
{
	for(int i=0; i<NUM_SHARED_SHADER_PARAMS;i++)
	{
		s_aShaderParameters[i] = 0;		
		if (s_aShaders[0].m_pD3DXEffect)
		{
			for(int j=0; j<NUM_SHADERS; j++)
			{				
				if (s_aShaderParameters[i] == 0)
				{
					s_aShaderParameters[i] = s_aShaders[j].m_pD3DXEffect->GetParameterByName(NULL, g_aszParamNames[i]);
				}
			}
		}
	}
}


void Platform_Shaders::ReleaseSharedShaderParams(void)
{
	for(int i=0; i<NUM_SHARED_SHADER_PARAMS;i++)
	{
		s_aShaderParameters[i] = 0;
	}
}


void Platform_Shaders::SetSharedParamMatrix44(SHADER_SHAREDPARAM eParam, Matrix& mtx)
{
	if (s_aShaderParameters[eParam])
	{
		s_aShaders[0].m_pD3DXEffect->SetMatrix(s_aShaderParameters[eParam], (D3DXMATRIX *)(&mtx));
	}
}


void Platform_Shaders::SetSharedParamVector4(SHADER_SHAREDPARAM eParam, Vector& vec)
{
	if (s_aShaderParameters[eParam])
	{
		s_aShaders[0].m_pD3DXEffect->SetVector(s_aShaderParameters[eParam], (D3DXVECTOR4 *)(&vec));
	}
}

void Platform_Shaders::SetSharedParamFloat(SHADER_SHAREDPARAM eParam, float fVal)
{
	if (s_aShaderParameters[eParam])
	{		
		s_aShaders[0].m_pD3DXEffect->SetFloat(s_aShaderParameters[eParam], fVal);		
	}
}


void Platform_Shaders::SetTexture(SHADER eShader, int iStage, Texture *pxTex)
{
	D3DShader *pxShader = &s_aShaders[eShader];
	
	if (pxShader->m_pD3DXEffect && pxShader->m_aTextureHandles[iStage])
	{
		if (pxTex)
			pxShader->m_pD3DXEffect->SetTexture(pxShader->m_aTextureHandles[iStage], pxTex->GetPlatformData()->m_pD3DTexture);
		else
			pxShader->m_pD3DXEffect->SetTexture(pxShader->m_aTextureHandles[iStage],NULL);
	}
}



void Platform_Shaders::Begin(SHADER eShader)
{
	if (s_iCurrentShader>=0)
	{
		End();
	}

	ID3DXEffect *pxEffect = s_aShaders[eShader].m_pD3DXEffect;

	if (pxEffect)
	{
		uint32 uPasses = 0;		

		if (s_aShaders[eShader].m_pD3DVertexDecl != s_pCurrentVertexDecl)
		{
			s_pCurrentVertexDecl = s_aShaders[eShader].m_pD3DVertexDecl;
			Platform_Render::GetD3DDevice()->SetVertexDeclaration(s_pCurrentVertexDecl);			
		}

		pxEffect->Begin(&uPasses, D3DXFX_DONOTSAVESTATE);
		pxEffect->BeginPass(0);

		s_iCurrentShader = eShader;
	}
}

void Platform_Shaders::End()
{
	if (s_iCurrentShader>=0)
	{
		ID3DXEffect *pxEffect = s_aShaders[s_iCurrentShader].m_pD3DXEffect;

		if (pxEffect)
		{
			pxEffect->EndPass();	
			pxEffect->End();	
		}

		s_iCurrentShader = -1;
	}
}