/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Platform_Shader Header (Windows)
* Author  : Dave Reed
*/

/*-Include-Once--------------------------------------------------------------*/

#if !defined(__BASE_PLATFORM_SHADERS_H__)
#define __BASE_PLATFORM_SHADERS_H__ /*!< \brief Include once define. */

/*-Project-Includes----------------------------------------------------------*/

#include "Base/Render.h"
#include "Base/Shaders.h"
#include "Base/Texture.h"
#include "Platform_Render.h"

/*-Defines-------------------------------------------------------------------*/

#define MAX_TEXTURE_STAGES 8


struct ShaderListEntry
{
	char			  *szShaderFile;
	D3DVERTEXELEMENT9 *pVertexDecl;
};


class D3DShader
{
public:
	D3DShader()
	{
		m_pD3DXEffect    = NULL;
		m_pD3DVertexDecl = NULL;
		for(int i=0; i<MAX_TEXTURE_STAGES; i++) m_aTextureHandles[i] = NULL;
	}

	ID3DXEffect					   *m_pD3DXEffect;
	IDirect3DVertexDeclaration9	   *m_pD3DVertexDecl;
	D3DXHANDLE						m_aTextureHandles[MAX_TEXTURE_STAGES];
};


class Platform_Shaders
{

	static ID3DXEffectPool				*s_pShaderPool;
	static D3DShader					 s_aShaders[NUM_SHADERS];
	static D3DXHANDLE					 s_aShaderParameters[NUM_SHARED_SHADER_PARAMS];
	static int							 s_iCurrentShader;
	static IDirect3DVertexDeclaration9	*s_pCurrentVertexDecl;

public:
	static void Initialize(void);
	static void Shutdown(void);

	static void PreReset(void);
	static void PostReset(void);

	static void GetSharedShaderParams(void);
	static void ReleaseSharedShaderParams(void);
	static void UpdateSharedShaderParameters(void);

	static void SetShader(SHADER eShader);
//	static void SetTexture(int iStage, Texture *pTex);	

	static void SetSharedParamMatrix44(SHADER_SHAREDPARAM eParam, Matrix& mtx);
	static void SetSharedParamVector4(SHADER_SHAREDPARAM eParam, Vector& vec);
	static void SetSharedParamFloat(SHADER_SHAREDPARAM eParam, float fVal);

	static void SetTexture(SHADER eShader, int iStage, Texture *pxTex);

	static void Begin(SHADER eShader);
	static void End(void);
};


#endif // __BASE_PLATFORM_SHADERS_H__ 