
/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Texture Source (PC)
* Author  : Dave Reed
*/

/*-Includes------------------------------------------------------------------*/

#include "Main.h"
#include "Base/Texture.h"
#include "Platform_Render.h"

/*-Classes-------------------------------------------------------------------*/

bool Platform_Texture::Load(Texture *pTex, bool bMipMaps)
{	
	char szFileName[MAX_FILENAME_LENGTH];	
	sprintf(szFileName, "data\\%s", pTex->m_szFileName);

	D3DXIMAGE_INFO Info;

	HRESULT hr = D3DXCreateTextureFromFileEx( Platform_Render::GetD3DDevice(), szFileName, D3DX_DEFAULT, D3DX_DEFAULT,
											  bMipMaps?D3DX_DEFAULT:D3DX_FROM_FILE, 0, D3DFMT_FROM_FILE, D3DPOOL_MANAGED,
											  D3DX_DEFAULT, D3DX_DEFAULT, 0, &Info, NULL, &pTex->GetPlatformData()->m_pD3DTexture);

	if (hr==D3D_OK)
	{
		pTex->m_iWidth  = Info.Width;
		pTex->m_iHeight = Info.Height;
		return true;
	}
	else
	{
		return false;
	}	
}


void Platform_Texture::CreateRenderTarget(Texture *pTex)
{
	PostReset(pTex);
}


void Platform_Texture::PreReset(Texture *pTex)
{
	if (pTex->m_bIsRenderTarget)
	{
		if (pTex->m_pPlatformData.m_pD3DTexture)
		{
			pTex->m_pPlatformData.m_pD3DTexture->Release();
			pTex->m_pPlatformData.m_pD3DTexture = NULL;
		}
	}
}


void Platform_Texture::PostReset(Texture *pTex)
{
	if (pTex->m_bIsRenderTarget)
	{
		// Create the rendertarget now
		if (pTex->m_uFlags & TEXTUREFLAG_SCREENSIZE)
		{
			pTex->m_iWidth  = Render::GetScreenWidth();
			pTex->m_iHeight = Render::GetScreenHeight();
		}

		if (pTex->m_uFlags & TEXTUREFLAG_HALFSCREENSIZE)
		{
			pTex->m_iWidth  = Render::GetScreenWidth()  / 2;
			pTex->m_iHeight = Render::GetScreenHeight() / 2;
		}

		if (pTex->m_iWidth <1) pTex->m_iWidth  = 1;
		if (pTex->m_iHeight<1) pTex->m_iHeight = 1;

		D3DFORMAT fmt = D3DFMT_A8R8G8B8;
		Platform_Render::GetD3DDevice()->CreateTexture(pTex->m_iWidth, pTex->m_iHeight, 1, D3DUSAGE_RENDERTARGET, fmt, D3DPOOL_DEFAULT, &pTex->m_pPlatformData.m_pD3DTexture , NULL);
	}
}



void Platform_Texture::Flush(Texture *pTex)
{
	Platform_Texture *pPlatformData = pTex->GetPlatformData();

	if (pPlatformData->m_pD3DTexture)
	{
		pPlatformData->m_pD3DTexture->Release();
		pPlatformData->m_pD3DTexture = NULL;
	}
}

	
void Platform_Texture::BeginScene(Texture *pTex)
{
	// Make current texture the render target, and set it's viewport
	IDirect3DSurface9 *pSurface = NULL;
	if (D3D_OK == pTex->m_pPlatformData.m_pD3DTexture->GetSurfaceLevel(0, &pSurface))
	{
		Platform_Render::GetD3DDevice()->SetRenderTarget(0, pSurface);
		pSurface->Release();
	}
}

void Platform_Texture::EndScene(Texture *pTex)
{
	// Render to the back buffer
	IDirect3DSurface9 *pSurface = NULL;
	if (D3D_OK == Platform_Render::GetD3DDevice()->GetBackBuffer(0,0, D3DBACKBUFFER_TYPE_MONO, &pSurface))
	{
		Platform_Render::GetD3DDevice()->SetRenderTarget(0, pSurface);
		pSurface->Release();
	}
}

