

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Base_Bloom.fxh
//
// Shader for extracting high-brightness areas for blooming
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include "Includes/Common.fxh"

// Global constants
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

TEXTURE		g_Texture0;


// Samplers
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

sampler2D Sampler0 = sampler_state
{
	Texture = (g_Texture0);
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;	
	AddressU  = CLAMP;
	AddressV  = CLAMP;	
};


// Vertex Structs
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

struct VSIn									
{
	float2 f2Position	: POSITION;			
	float4 f4Colour		: COLOR0;			
	float2 f2UV		    : TEXCOORD0;		
};

struct VSOut 
{
	float4 f4Position	: POSITION;	
   	float2 f2UV			: TEXCOORD0;   	   	
   	float4 f4Colour		: COLOR0;
};


// Vertex Shader
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

VSOut VS(VSIn vsIn) 
{
	VSOut vsOut;

   	// Transform vertex by our 2D transform matrix
   	float4 f4Pos         = mul (float4(vsIn.f2Position,0,1), g_f4x42DXform);	
   	
   	// Scale pixel coords to normalized viewport coords
	vsOut.f4Position.x   = f4Pos.x * g_f4PixelsToViewport.x + g_f4PixelsToViewport.z;
	vsOut.f4Position.y   = f4Pos.y * g_f4PixelsToViewport.y + g_f4PixelsToViewport.w;
	vsOut.f4Position.z   = 0.5f;
	vsOut.f4Position.w   = 1.0f;
	vsOut.f4Colour		 = vsIn.f4Colour;		
	vsOut.f2UV		 	 = vsIn.f2UV;
	return vsOut; 
}


// Pixel Shader
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

struct PSOut
{
	float4 f4Colour		: COLOR0;
};

PSOut PS(VSOut psIn)
{	
	PSOut psOut;	
	
	float4 fCol = tex2D(Sampler0, psIn.f2UV);					 				
	
	fCol = (fCol - g_f4EffectControl.x) * g_f4EffectControl.y;
	
	float3 fMono = max(fCol.r,fCol.g); //(fCol.r+fCol.g+fCol.b) * 0.333f;
	fMono = max(fMono,fCol.b);
	
	psOut.f4Colour.rgb =  psIn.f4Colour * lerp(fCol.rgb, fMono, g_f4EffectControl.b);
	psOut.f4Colour.a   =  1;
	
	return psOut;
}


// Techniques.  
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

technique Shader_2_0
{
    pass Default
    {		
		VertexShader	 = compile vs_2_0 VS();
		PixelShader		 = compile ps_2_0 PS();
    }
}
