

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Base_Blur.fxh
//
// Blur shader for postproc effects
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include "Includes/Common.fxh"

// Global constants
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

TEXTURE		g_Texture0;


// Samplers
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

sampler2D Sampler0 = sampler_state
{
	Texture = (g_Texture0);
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	AddressU  = CLAMP;
	AddressV  = CLAMP;		
};


// Vertex Structs
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

struct VSIn									
{
	float2 f2Position	: POSITION;			
	float4 f4Colour		: COLOR0;			
	float2 f2UV		    : TEXCOORD0;		
};

struct VSOut 
{
	float4 f4Position	: POSITION;	
   	float2 af2UV[7]		: TEXCOORD0;   	   	
   	float4 f4Colour		: COLOR0;
};


// Vertex Shader
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

VSOut VS(VSIn vsIn) 
{
	VSOut vsOut;

   	// Transform vertex by our 2D transform matrix
   	float4 f4Pos         = mul (float4(vsIn.f2Position,0,1), g_f4x42DXform);	
   	
   	// Scale pixel coords to normalized viewport coords
	vsOut.f4Position.x   = f4Pos.x * g_f4PixelsToViewport.x + g_f4PixelsToViewport.z;
	vsOut.f4Position.y   = f4Pos.y * g_f4PixelsToViewport.y + g_f4PixelsToViewport.w;
	vsOut.f4Position.z   = 0.5f; //vsIn.f3Position.z;
	vsOut.f4Position.w   = 1.0f;
	vsOut.f4Colour		 = vsIn.f4Colour;		
	
	float2 f2Offset;
	f2Offset.xy = g_f4EffectControl.xy;	
	
	vsOut.af2UV[0]	 	 = vsIn.f2UV + (-3 * f2Offset );
	vsOut.af2UV[1]	 	 = vsIn.f2UV + (-2 * f2Offset );
	vsOut.af2UV[2]	 	 = vsIn.f2UV + (-1 * f2Offset );
	vsOut.af2UV[3]	 	 = vsIn.f2UV + ( 0 * f2Offset );
	vsOut.af2UV[4]	 	 = vsIn.f2UV + ( 1 * f2Offset );
	vsOut.af2UV[5]	 	 = vsIn.f2UV + ( 2 * f2Offset );
	vsOut.af2UV[6]	 	 = vsIn.f2UV + ( 3 * f2Offset );
	
	return vsOut; 
}


// Pixel Shader
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

struct PSOut
{
	float4 f4Colour		: COLOR0;
};


static const float BlurWeights[7] = 
{	
	1.0f  / 66.0f,
	4.0f  / 66.0f,
	16.0f / 66.0f,
	24.0f / 66.0f,
	16.0f / 66.0f,
	4.0f  / 66.0f,
	1.0f  / 66.0f
}; 


PSOut PS(VSOut psIn)
{	
	PSOut psOut;	
	psOut.f4Colour =  psIn.f4Colour * g_f4EffectControl.w * 
	                 ( tex2D(Sampler0, psIn.af2UV[0]) * BlurWeights[0] +
	                   tex2D(Sampler0, psIn.af2UV[1]) * BlurWeights[1] +
	                   tex2D(Sampler0, psIn.af2UV[2]) * BlurWeights[2] +
	                   tex2D(Sampler0, psIn.af2UV[3]) * BlurWeights[3] +
	                   tex2D(Sampler0, psIn.af2UV[4]) * BlurWeights[4] +
	                   tex2D(Sampler0, psIn.af2UV[5]) * BlurWeights[5] +	
	                   tex2D(Sampler0, psIn.af2UV[6]) * BlurWeights[6] );	                   		                   	
	return psOut;
}


// Techniques.  
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

technique Shader_2_0
{
    pass Default
    {		
		VertexShader	 = compile vs_2_0 VS();
		PixelShader		 = compile ps_2_0 PS();
    }
}
