

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// PPR_Blitter3D_Lit.fxh
//
// Shader for standard 3D blitting , modulated by the lighting map
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include "Includes/Common.fxh"


// Global constants
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

TEXTURE g_Texture0;
TEXTURE g_Texture1;

// Samplers
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

sampler2D Sampler0 : register(s0) = sampler_state
{
	Texture = (g_Texture0);
	MipFilter	  = LINEAR;
	MinFilter	  = LINEAR;
	MagFilter	  = LINEAR;		
	AddressU	  = WRAP;
	AddressV	  = WRAP;	
};

sampler2D Sampler1 : register(s1) = sampler_state
{
	Texture = (g_Texture1);
	MipFilter	  = LINEAR;
	MinFilter	  = LINEAR;
	MagFilter	  = LINEAR;		
	AddressU	  = CLAMP;
	AddressV	  = CLAMP;	
};

// Vertex Structs.                 
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

struct VSIn										
{
	float3 f3Position	: POSITION;				
	float2 f2UV		    : TEXCOORD0;			
	float4 f4Colour		: COLOR0;				
};

struct VSOut 
{
	float4 f4Position		: POSITION;	
	float4 f4ScrnPos		: TEXCOORD1;	
   	float2 f2UV			: TEXCOORD0;   	   	
   	float4 f4Colour		: COLOR0;   	   	
};


// Vertex Shader
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

VSOut VS(VSIn vsIn) 
{
	VSOut vsOut;
   	
	float4 pos = mul (float4(vsIn.f3Position,1), g_f4x4WorldViewProj);	
	vsOut.f4ScrnPos		 = pos;
	vsOut.f4Position     	 = pos; 
	vsOut.f4Colour		 = vsIn.f4Colour;				
	vsOut.f2UV			 = vsIn.f2UV;

	return vsOut;
}


// Pixel Shader
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

struct PSOut
{
	float4 f4Colour		: COLOR0;
};

PSOut PS(VSOut psIn)
{	
	PSOut psOut;	

	// Calc UVs in lighting map
	float2 f2LightMapUV = 0.5f + (psIn.f4ScrnPos.xy / psIn.f4ScrnPos.w) * float2(0.5f, -0.5f);

	float4 f4Lighting = tex2D(Sampler1, f2LightMapUV);
	float4 f4Unlit    = psIn.f4Colour * tex2D(Sampler0, psIn.f2UV);							
	
	psOut.f4Colour.rgb = lerp(f4Unlit.rgb, 2.0f * f4Unlit.rgb * f4Lighting.rgb, g_fLightingAmount); //+ f4Specular.rgb*f4Unlit.rgb;
	psOut.f4Colour.a   = f4Unlit.a;

	return psOut;
}


// Techniques
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

technique Shader_2_0
{
    pass Default
    {			
		VertexShader	 = compile vs_2_0 VS();
		PixelShader		 = compile ps_2_0 PS();
    }
}

