

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// PPR_Blitter3D_Lit.fxh
//
// Shader for standard 3D blitting , modulated by the lighting map
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include "Includes/Common.fxh"


// Global constants
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

TEXTURE g_Texture0;
TEXTURE g_Texture1;
TEXTURE g_Texture2;

// Samplers
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

sampler2D Sampler0 : register(s0) = sampler_state
{
	Texture = (g_Texture0);
	MipFilter	  = LINEAR;
	MinFilter	  = LINEAR;
	MagFilter	  = LINEAR;		
	AddressU	  = WRAP;
	AddressV	  = WRAP;	
};

sampler2D Sampler1 : register(s1) = sampler_state
{
	Texture = (g_Texture1);
	MipFilter	  = LINEAR;
	MinFilter	  = LINEAR;
	MagFilter	  = LINEAR;		
	AddressU	  = CLAMP;
	AddressV	  = CLAMP;	
};

sampler2D Sampler2 : register(s2) = sampler_state
{
	Texture = (g_Texture2);
	MipFilter	  = LINEAR;
	MinFilter	  = LINEAR;
	MagFilter	  = LINEAR;		
	AddressU	  = WRAP;
	AddressV	  = WRAP;	
};

// Vertex Structs.                 
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

struct VSIn										
{
	float3 f3Position	: POSITION;				
	float2 f2UV		    : TEXCOORD0;			
	float4 f4Colour		: COLOR0;				
};

struct VSOut 
{
	float4 f4Position		: POSITION;	
	float4 f4ScrnPos		: TEXCOORD1;	
   	float2 f2UV				: TEXCOORD0;   	   	
   	float4 f4Colour			: COLOR0;   	 
   	float2 f2SplitY			: TEXCOORD2;  	
};


// Vertex Shader
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

VSOut VS(VSIn vsIn) 
{
	VSOut vsOut;
   	
	float4 pos = mul (float4(vsIn.f3Position,1), g_f4x4WorldViewProj);	
	vsOut.f4ScrnPos		 = pos;
	vsOut.f4Position     = pos; 
	vsOut.f4Colour		 = vsIn.f4Colour;				
	vsOut.f2UV			 = vsIn.f2UV;

	float4 f4Temp       = float4(0,g_f4EffectControl.x,0,1);
   	float4 f4Out		= mul(f4Temp, g_f4x4WorldViewProj);	   	
   	float4 fTop			= f4Out.y;
   	f4Temp		 		= float4(0,g_f4EffectControl.y,0,1);
   	f4Out				= mul(f4Temp, g_f4x4WorldViewProj);	   	
   	float fBot			= f4Out.y;	
		
	vsOut.f2SplitY.x        = 0.5f + (fTop / vsOut.f4ScrnPos.w) * -0.5f;	
	vsOut.f2SplitY.y        = 0.5f + (fBot / vsOut.f4ScrnPos.w) * -0.5f;	
	vsOut.f2SplitY.x-=vsOut.f2SplitY.y;
	
	return vsOut;
}


// Pixel Shader
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

struct PSOut
{
	float4 f4Colour		: COLOR0;
};

PSOut PS(VSOut psIn)
{	
	PSOut psOut;	

	// Calc UVs in lighting map
	float2 f2LightMapUV = 0.5f + (psIn.f4ScrnPos.xy / psIn.f4ScrnPos.w) * float2(0.5f, -0.5f);

	f2LightMapUV.y = psIn.f2SplitY.y + (psIn.f2SplitY.x*psIn.f2UV.y);

	float2 f2DistortUV = psIn.f2UV * 0.4f;
	f2DistortUV.y+= g_f4EffectControl.z * 0.05f;
	f2DistortUV.x+= g_f4EffectControl.z * 0.025f;
	float2 f2Distort =   (psIn.f2UV.y+0.2f)*(0.1f*  (tex2D(Sampler2, f2DistortUV).rg-0.5f) );
	
	f2DistortUV = psIn.f2UV * 1.0f;
	f2DistortUV.y-= g_f4EffectControl.z * 0.003f;
	f2DistortUV.x-= g_f4EffectControl.z * 0.0012f;
	f2Distort +=   psIn.f2UV.y*(0.05f*  (tex2D(Sampler2, f2DistortUV).rg-0.5f) );
	

//f2Distort.y=f2Distort.x;
//f2Distort.x=0;

	float4 f4Lighting = tex2D(Sampler0, f2LightMapUV + f2Distort);
	
	psOut.f4Colour.rgb = lerp(f4Lighting.rgb, 0.333f*(f4Lighting.r+f4Lighting.r+f4Lighting.b), 0.8f);	
	psOut.f4Colour.rgb = float3(0.05,0.1,0.12)+psOut.f4Colour.rgb * psIn.f4Colour;
	
	psOut.f4Colour.a   = tex2D(Sampler1, psIn.f2UV).r;

	return psOut;
}


// Techniques
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

technique Shader_2_0
{
    pass Default
    {			
		VertexShader	 = compile vs_2_0 VS();
		PixelShader		 = compile ps_2_0 PS();
    }
}

