

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Blitter3D.fxh
//
// Shader for bog standard standard 3D blitting 
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include "Includes/Common.fxh"


// Global constants
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

TEXTURE g_Texture0;
TEXTURE g_Texture1;


// Samplers
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

sampler2D Sampler0 : register(s0) = sampler_state
{
	Texture = (g_Texture0);
	MipFilter	  = LINEAR;
	MinFilter	  = LINEAR;
	MagFilter	  = LINEAR;		
	AddressU	  = WRAP;
	AddressV	  = WRAP;	
};


sampler2D Sampler1 : register(s1) = sampler_state
{
	Texture = (g_Texture1);
	MipFilter	  = LINEAR;
	MinFilter	  = LINEAR;
	MagFilter	  = LINEAR;		
	AddressU	  = CLAMP;
	AddressV	  = CLAMP;	
};


// Vertex Structs.                 
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

struct VSIn										
{
	float3 f3Position	: POSITION;				
	float2 f2UV		    : TEXCOORD0;			
	float4 f4Colour		: COLOR0;	
	float3 f3Normal		: NORMAL;			
};

struct VSOut 
{
	float4 f4Position	: POSITION;	
   	float2 f2UV			: TEXCOORD0;   	   	
   	float2 f2UV2		: TEXCOORD1;   	   	
   	float4 f4Colour		: COLOR0;   	   	   	
};


// Vertex Shader
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

VSOut VS(VSIn vsIn) 
{
	VSOut vsOut;
   	
	vsOut.f4Position     = mul (float4(vsIn.f3Position,1), g_f4x4WorldViewProj);	
		
	vsOut.f4Colour		 = vsIn.f4Colour;				
	vsOut.f2UV			 = vsIn.f2UV;
	vsOut.f2UV2          = 0.5f+ ((vsIn.f3Position.xz - g_f4EffectControl.xz) * g_f4EffectControl.w);

	return vsOut;
}


// Pixel Shader
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

struct PSOut
{
	float4 f4Colour		: COLOR0;
};

PSOut PS(VSOut psIn)
{	
	PSOut psOut;	
	psOut.f4Colour = psIn.f4Colour * tex2D(Sampler0, psIn.f2UV) * tex2D(Sampler1, psIn.f2UV2);							
	return psOut;
}


// Techniques
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

technique Shader_2_0
{
    pass Default
    {			
		VertexShader	 = compile vs_2_0 VS();
		PixelShader		 = compile ps_2_0 PS();
    }
}

