
/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Postprocess Source (Independent)
* Author  : Dave Reed
*/

#include "Main.h"
#include "Base/Render.h"
#include "Base/PostProcess.h"

Texture *PostProcess::s_pMainOffScreenBuffer;		// Full resolution off-screen buffer
Texture *PostProcess::s_apBlurBuffers[2];			// Half-resolution blur/bloom buffer


void PostProcess::Initialize(void)
{
	s_pMainOffScreenBuffer = Texture::CreateRenderTarget(0, 0, TEXTUREFLAG_SCREENSIZE);
	s_apBlurBuffers[0]	   = Texture::CreateRenderTarget(0, 0, TEXTUREFLAG_HALFSCREENSIZE);
	s_apBlurBuffers[1]	   = Texture::CreateRenderTarget(0, 0, TEXTUREFLAG_HALFSCREENSIZE);
}



void PostProcess::Shutdown(void)
{
	Texture::FlushTexture( s_apBlurBuffers[1] );
	Texture::FlushTexture( s_apBlurBuffers[0] );
	Texture::FlushTexture( s_pMainOffScreenBuffer );
}



void PostProcess::BeginScene(void)
{
	s_pMainOffScreenBuffer->BeginScene();
}



void PostProcess::EndScene(bool bBlitToBackBuffer)
{
	s_pMainOffScreenBuffer->EndScene();

	if (bBlitToBackBuffer)
	{
		BlitRect2D xRect;

		xRect.fX[0] = 0.0f;
		xRect.fX[1] = Render::Get2DScreenRightX();
		xRect.fY[0] = 0.0f;
		xRect.fY[1] = Render::Get2DScreenBottomY();
		xRect.fU[0] = 0.0f; xRect.fU[1] = 1.0f;
		xRect.fV[0] = 0.0f; xRect.fV[1] = 1.0f;
		xRect.uARGB = 0xffffffff;

		RenderStates::SetBlendMode( RS_BLEND_NONE );
		Blitter2D::BlitRects(1, &xRect, s_pMainOffScreenBuffer);	
	}
}


void PostProcess::ApplyBloom()
{	
	// First, extract the areas we want to bloom (for now, copy the whole thing)
	s_pMainOffScreenBuffer->EndScene();

	RenderStates::SetBlendMode( RS_BLEND_NONE );		

	{
		s_apBlurBuffers[0]->BeginScene();	

		BlitRect2D xRect;
		xRect.uARGB = 0xffffffff;
		xRect.fX[0] = 0.0f;   xRect.fX[1] = Render::Get2DScreenRightX();
		xRect.fY[0] = 0.0f;   xRect.fY[1] = Render::Get2DScreenBottomY();
		xRect.fU[0] = 0.0f;   xRect.fU[1] = 1.0f;
		xRect.fV[0] = 0.0f;   xRect.fV[1] = 1.0f;		

		Vector vEffectControl = Vector(0.6f, 3.0f, 0.65f, 0.0f);
		Shaders::SetSharedParamVector4(SHADER_PARAM_EFFECTCONTROL, vEffectControl);

		Blitter2D::BlitRects(1, &xRect, s_pMainOffScreenBuffer, SHADER_BLOOM);	
		
		s_apBlurBuffers[0]->EndScene();
	}

	Blur(3, 1.0f, 1.1f);

	// Blit it back over the main back buffer	
	{
		s_pMainOffScreenBuffer->BeginScene();

		RenderStates::SetBlendMode( RS_BLEND_ADD );		

		BlitRect2D xRect;
		xRect.uARGB = 0xffffffff; 
		xRect.fX[0] = 0.0f;   xRect.fX[1] = Render::Get2DScreenRightX();
		xRect.fY[0] = 0.0f;   xRect.fY[1] = Render::Get2DScreenBottomY();
		xRect.fU[0] = 0.0f;   xRect.fU[1] = 1.0f;
		xRect.fV[0] = 0.0f;   xRect.fV[1] = 1.0f;		
		
		Blitter2D::BlitRects(1, &xRect, s_apBlurBuffers[0]);	
	}
	
}


void PostProcess::Blur(int iNumPasses, float fSizeScale, float fDividerScale)
{

	// Repeatedly blur it between the buffers
	for(int j=0; j<iNumPasses; j++)
	{
		for(int i=0; i<2; i++)
		{
			s_apBlurBuffers[1-i]->BeginScene();

			BlitRect2D xRect;
			xRect.uARGB = 0xffffffff;
			xRect.fX[0] = 0.0f;   xRect.fX[1] = Render::Get2DScreenRightX();
			xRect.fY[0] = 0.0f;   xRect.fY[1] = Render::Get2DScreenBottomY();
			xRect.fU[0] = 0.0f;   xRect.fU[1] = 1.0f;
			xRect.fV[0] = 0.0f;   xRect.fV[1] = 1.0f;	

			Vector vEffectControl = Vector(0.0f, 0.0f, 0.0f, fDividerScale);
			if (i==0) vEffectControl.m_fX = fSizeScale * 0.002f * (float)(j+1);
			if (i==1) vEffectControl.m_fY = fSizeScale * 0.002f * (float)(j+1);

			Shaders::SetSharedParamVector4(SHADER_PARAM_EFFECTCONTROL, vEffectControl);

			Blitter2D::BlitRects(1, &xRect, s_apBlurBuffers[i], SHADER_BLUR);			

			s_apBlurBuffers[1-i]->EndScene();
		}
	}
}