/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Postprocess Header (Independent)
* Author  : Dave Reed
*/

/*-Include-Once--------------------------------------------------------------*/

#if !defined(__BASE_POSTPROCESS_H__)
#define __BASE_POSTPROCESS_H__ /*!< \brief Include once define. */

#include "Base/Viewport.h"
#include "Base/Camera.h"
#include "Base/Shaders.h"
#include "Base/RenderStates.h"
#include "Base/Blitter.h"

/*-Classes-------------------------------------------------------------------*/

class PostProcess
{
public:
	static void Initialize(void);							// Init the render targets
	static void Shutdown(void);								// Release the render targets

	static void BeginScene(void);							// Start rendering to the off-screen buffer
	static void EndScene(bool bBlitToBackBuffer = true);	// Blit offscreen buffer back to real back buffer, and restore main render targets

	static void ApplyBloom(void);	

	static void Blur(int iNumPasses = 3, float fSizeScale = 1.0f, float fDividerScale = 1.0f);

	static Texture *GetMainOffScreenBuffer(void) {return s_pMainOffScreenBuffer;}

private:

	static Texture *s_pMainOffScreenBuffer;					// Full resolution off-screen buffer
	static Texture *s_apBlurBuffers[2];						// Half-resolution blur/bloom buffer
};


#endif /* __BASE_POSTPROCESS_H__ */
