/*! \file
 * Copyright (c) 2008
 * Project : Base
 * File    : Render Header (Independent)
 * Author  : Dave Reed
 */

/*-Include-Once--------------------------------------------------------------*/

#if !defined(__BASE_RENDER_H__)
#define __BASE_RENDER_H__ /*!< \brief Include once define. */

#include "Base/Viewport.h"
#include "Base/Camera.h"
#include "Base/Shaders.h"
#include "Base/RenderStates.h"
#include "Base/Blitter.h"

/*-Classes-------------------------------------------------------------------*/

class RenderConfig
{
public:

	RenderConfig()
	{
		m_iWindowedWidth = 1024;
		m_iWindowedHeight = 600;
		m_iFullScreenWidth = 1024;
		m_iFullScreenHeight = 768;
		m_iMultisample = 0;
		m_bWindowed = true;
		m_uClearARGB = 0x808080;
		m_szProcessName = "LD48 #13";
		m_szWindowTitle = "LD48 #13 [Alt+Enter for fullscreen mode]";
		m_hWindowHandle = NULL;
	}

	int		m_iWindowedWidth;
	int		m_iWindowedHeight;
	int		m_iFullScreenWidth;
	int		m_iFullScreenHeight;
	int		m_iMultisample;
	bool	m_bWindowed;
	char   *m_szProcessName;
	char   *m_szWindowTitle;
	uint32	m_uClearARGB;
	HWND	m_hWindowHandle;
};

class Render
{
public:
	
	static bool  Initialize(RenderConfig &xConfig);
	static void  Shutdown(void);
	static void  Reset(void);
	static void	 PresentFrame(void);
	static void  ClearViewport(uint32 uCol, bool bColor, bool bDepth, bool bStencil);
	static bool  IsExitRequested(void);
	static int	 GetScreenWidth(void);
	static int	 GetScreenHeight(void);
	static int	 GetViewportWidth(void);
	static int	 GetViewportHeight(void);
	static bool	 IsRenderingPaused(void);
	static bool	 IsRenderingFocused(void);
	static float GetAspectRatio(void);

	static void  Reset2DScaling();
	static void  Set2DVirtualCoords(float fHeight);
	static void  Reset2DTransform();
	static void  Set2DTransform(float fX, float fY, float fRotation, float fXScale, float fYScale);

	static float Get2DScreenLeftX(); 
	static float Get2DScreenRightX();
	static float Get2DScreenCentreX();

	static float Get2DScreenTopY(); 
	static float Get2DScreenBottomY();
	static float Get2DScreenMiddleY();

	static void SetWorldMatrix(Matrix& mtx);
	


	static void  Flush(void);

	static RenderConfig& GetCurrentConfig(void) {return s_xCurrentConfig;}

protected:
    
	static float		s_f2DVirtualScreenWidth;
	static float		s_f2DVirtualScreenHeight;

	static RenderConfig s_xCurrentConfig;		
};

#endif /* __BASE_RENDER_H__ */
