/*! \file
* Copyright (c) 2008
* Project : Base
* File    : RenderStates Header (Independent)
* Author  : Dave Reed
*/

/*-Include-Once--------------------------------------------------------------*/

#if !defined(__BASE_RENDERSTATES_H__)
#define __BASE_RENDERSTATES_H__ 

/*-Classes-------------------------------------------------------------------*/

typedef enum RS_CULL_MODE
{
	RS_CULL_NONE,
	RS_CULL_CW,
	RS_CULL_CCW
};

typedef enum RS_BLEND_MODE
{		
	RS_BLEND_NONE,				
	RS_BLEND_NORMAL,				// srcalpha,  invsrcalpha
	RS_BLEND_ADD,					// srcalpha,  one
	RS_BLEND_MULTIPLY				// dstcolour, zero
};

typedef enum RS_ZBUFFER_MODE
{
	RS_ZBUFFER_OFF,
	RS_ZBUFFER_LESSEQUAL,
	RS_ZBUFFER_GREATEREQUAL
};

class RenderStates
{
public:		

	// Just a very thin wrapper around platform calls at the moment. I'm not going to attempt any caching of renderstates,
	// as that causes all sorts of problems if renderstates get changed directly, via .fx files, for example

	// Depth Buffer states
	inline static void SetZBufferMode( RS_ZBUFFER_MODE eMode )										{Platform_SetZBufferMode( eMode );}
	inline static void SetZWriteEnable( bool bEnable )												{Platform_SetZWriteEnable( bEnable );}

	// Alpha Blending states			
	inline static void SetAlphaTest( bool bEnable, int iThreshold )									{Platform_SetAlphaTest( bEnable, iThreshold );}
	inline static void SetBlendMode( RS_BLEND_MODE eMode )											{Platform_SetBlendMode( eMode );}

	// Cullng states
	inline static void SetCullMode( RS_CULL_MODE eMode )											{Platform_SetCullMode( eMode );}

	// Misc states
	inline static void SetColorWriteEnable( bool bRed, bool bGreen, bool bBlue, bool bAlpha )		{SetColorWriteEnable( bRed, bGreen, bBlue, bAlpha );}

private:

	// Depth Buffer states
	static void Platform_SetZBufferMode( RS_ZBUFFER_MODE eMode );									
	static void Platform_SetZWriteEnable( bool bEnable );

	// Alpha Blending states			
	static void Platform_SetAlphaTest( bool bEnable, int iThreshold );
	static void Platform_SetBlendMode( RS_BLEND_MODE eMode );

	// Cullng states
	static void Platform_SetCullMode( RS_CULL_MODE eMode );

	// Misc states
	static void Platform_SetColorWriteEnable( bool bRed, bool bGreen, bool bBlue, bool bAlpha );

};


#endif /* __BASE_RENDERSTATES_H__ */
