
/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Shaders Header (Independent)
* Author  : Dave Reed
*/

/*-Include-Once--------------------------------------------------------------*/

#if !defined(__BASE_SHADERS_H__)
#define __BASE_SHADERS_H__ /*!< \brief Include once define. */

#include "Base/Texture.h"

/*-Shared Shader Params------------------------------------------------------*/

typedef enum SHADER_SHAREDPARAM
{		
	SHADER_PARAM_WORLDVIEWPROJ = 0,	
	SHADER_PARAM_2DXFORM,
	SHADER_PARAM_PIXELSTOVIEWPORT,
	SHADER_PARAM_EFFECTCONTROL,
	SHADER_PARAM_LIGHTINGSTRENGTH,
	NUM_SHARED_SHADER_PARAMS
};


/*-Vertex Formats------------------------------------------------------------*/

struct Vertex2D
{
	float	fX, fY; 
	float	fU, fV;
	uint32  uARGB;
};


struct Vertex3D
{
	float	fX, fY, fZ;			
	float	fU, fV;
	uint32  uARGB;
};


struct Vertex3DLit
{
	float	fX, fY, fZ;			
	float	fU, fV;
	uint32  uARGB;
	float   fNX, fNY, fNZ;
};

/*-Classes-------------------------------------------------------------------*/

typedef enum SHADER
{
	SHADER_BLITTER2D,
	SHADER_BLITTER3D,
	SHADER_BLOOM,
	SHADER_BLUR,	
	SHADER_BLITTER3D_LIT,
	NUM_SHADERS
};


class Shaders
{
public:

	static void Initialize(void);
	static void Shutdown(void);

	static void SetTexture(SHADER eShader, int iStage, Texture *pxTex);
	static void SetSharedParamMatrix44(SHADER_SHAREDPARAM eParam, Matrix& mtx);
	static void SetSharedParamVector4(SHADER_SHAREDPARAM eParam, Vector& vec);
	static void SetSharedParamFloat(SHADER_SHAREDPARAM eParam, float f);
};

#endif // __BASE_SHADERS_H__