
/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Texture Source (Independent)
* Author  : Dave Reed
*/


/*-Includes------------------------------------------------------------------*/

#include "Main.h"
#include "Base/Texture.h"
#include "Base/Render.h"

/*-Static Members------------------------------------------------------------*/

Texture  Texture::s_aTextures[MAX_TEXTURES];
Texture *Texture::s_apBaseTextures[NUM_BASE_TEXTURES];	

char *g_aszBaseTextureFileNames[NUM_BASE_TEXTURES] =
{
	"base\\textures\\white.tga",
	"base\\textures\\missing.tga"
};


void Texture::Initialize()
{
	for(int i=0; i<MAX_TEXTURES; i++)
	{
		Texture *pTex = &s_aTextures[i];
		pTex->m_iWidth = pTex->m_iHeight = pTex->m_iGroup = 0;
		pTex->m_bUsed = false;
		memset(pTex->m_szFileName,0, MAX_FILENAME_LENGTH);
	}

	for(int i=0; i<NUM_BASE_TEXTURES; i++)
	{
		s_apBaseTextures[i] = LoadTexture(g_aszBaseTextureFileNames[i]);
	}
}


void Texture::Shutdown()
{
	for(int i=0; i<MAX_TEXTURES; i++)
	{
		if (s_aTextures[i].m_bUsed)
		{
			FlushTexture(&s_aTextures[i]);
		}
	}
}


void Texture::PreReset()
{
	for(int i=0; i<MAX_TEXTURES; i++)
	{
		if (s_aTextures[i].m_bUsed)
		{
			Platform_Texture::PreReset(&s_aTextures[i]);
		}
	}
}


void Texture::PostReset()
{
	for(int i=0; i<MAX_TEXTURES; i++)
	{
		if (s_aTextures[i].m_bUsed)
		{
			Platform_Texture::PostReset(&s_aTextures[i]);
		}
	}
}


Texture *Texture::LoadTexture(char *szFileNameInc, bool bMipMaps, int iTextureGroup)
{
	int iFirstFreeSlot = -1;

	char szFileName[MAX_FILENAME_LENGTH];
	strcpy(szFileName, szFileNameInc);
	_strlwr(szFileName);

	for(int i=0; i<MAX_TEXTURES; i++)
	{
		if (s_aTextures[i].m_bUsed)
		{			
			if (strcmp(s_aTextures[i].m_szFileName, szFileName) == 0)
			{
				return &s_aTextures[i];
			}
		}
		else
		{
			if (iFirstFreeSlot<0)
			{
				iFirstFreeSlot = i;
			}
		}
	}


	if (iFirstFreeSlot>=0)
	{
		// Got a free slot, load it
		Texture *pTex = &s_aTextures[iFirstFreeSlot];

		strcpy(pTex->m_szFileName, szFileName);

		if (Platform_Texture::Load(pTex, bMipMaps))
		{
			pTex->m_bUsed   = true;
			pTex->m_bIsRenderTarget = false;
			pTex->m_uFlags  = 0;
			pTex->m_eFormat = TEXTUREFORMAT_UNKNOWN;
			return pTex;
		}
		else
		{
			memset(pTex->m_szFileName,0, MAX_FILENAME_LENGTH);
			return NULL;
		}			 
	}
	else
	{
		// Out of texture slots!
		return NULL;
	}
}



Texture *Texture::CreateRenderTarget(int iWidth, int iHeight, uint32 uFlags, TEXTUREFORMAT eFormat)
{
	int iFirstFreeSlot = -1;

	for(int i=0; i<MAX_TEXTURES; i++)
	{
		if (!s_aTextures[i].m_bUsed)
		{
			if (iFirstFreeSlot<0)
			{
				iFirstFreeSlot = i;
			}
		}
	}

	if (iFirstFreeSlot>=0)
	{
		// Got a free slot, load it
		Texture *pTex = &s_aTextures[iFirstFreeSlot];
		strcpy(pTex->m_szFileName,"rendertarget");

		pTex->m_bUsed   = true;
		pTex->m_uFlags  = uFlags;
		pTex->m_eFormat = eFormat;
		pTex->m_bIsRenderTarget = true;
		pTex->m_iWidth  = iWidth;
		pTex->m_iHeight = iHeight;

		Platform_Texture::CreateRenderTarget(pTex);
		return pTex;
	}
	else
	{
		return NULL;
	}
}



void Texture::FlushTexture(Texture* pTex)
{
	if (pTex->m_bUsed)
	{
		Platform_Texture::Flush(pTex);
		pTex->m_iWidth = pTex->m_iHeight = pTex->m_iGroup = 0;
		pTex->m_bUsed = false;
		memset(pTex->m_szFileName,0, MAX_FILENAME_LENGTH);
	}
}


void Texture::FlushTextures(int iTextureGroup)
{
	for(int i=0; i<MAX_TEXTURES; i++)
	{
		if (s_aTextures[i].m_bUsed && s_aTextures[i].m_iGroup==iTextureGroup)
		{
			FlushTexture(&s_aTextures[i]);
		}
	}
}

void Texture::BeginScene(void)
{
	if (m_bIsRenderTarget)
	{
		Platform_Texture::BeginScene(this);
		Render::Reset2DTransform();
	}
}

void Texture::EndScene(void)
{
	if (m_bIsRenderTarget)
	{
		Platform_Texture::EndScene(this);
	}
}