
/*! \file
* Copyright (c) 2008
* Project : Base
* File    : Texture Header (Independent)
* Author  : Dave Reed
*/

/*-Include-Once--------------------------------------------------------------*/

#if !defined(__BASE_TEXTURE_H__)
#define __BASE_TEXTURE_H__ /*!< \brief Include once define. */

#include "Base/Viewport.h"
#include "Base/Camera.h"
#include "Platform_Texture.h"

/*-Classes-------------------------------------------------------------------*/

#define MAX_TEXTURES 512


typedef enum BASETEXTURE
{
	BASE_TEXTURE_WHITE,
	BASE_TEXTURE_MISSING,
	NUM_BASE_TEXTURES
};

// Flags for rendertargets
#define TEXTUREFLAG_SCREENSIZE		1
#define TEXTUREFLAG_HALFSCREENSIZE 2

// Formats for rendertargets
enum TEXTUREFORMAT
{
	TEXTUREFORMAT_UNKNOWN   = 0,
	TEXTUREFORMAT_A8R8G8B8	= 1
};

class Texture
{
	friend class	Platform_Texture;

public:	

	// Static Texture Manager functions
	static void		Initialize();
	static void		Shutdown();
	static void     PreReset();
	static void     PostReset();

	static Texture *LoadTexture(char *szFileName, bool bMipMaps = true, int iTextureGroup = 0);
	
	static void		FlushTexture(Texture* pTex);
	static void		FlushTextures(int iTextureGroup);	
	static Texture *GetBaseTexture(BASETEXTURE eTex) {return s_apBaseTextures[eTex];}

	// Render-to-texture support
	static Texture *CreateRenderTarget(int iWidth, int iHeight, uint32 uFlags = 0, TEXTUREFORMAT eFormat = TEXTUREFORMAT_A8R8G8B8);

	static Texture  s_aTextures[MAX_TEXTURES];
	static Texture *s_apBaseTextures[NUM_BASE_TEXTURES];	

	// The Texture class itself
	Platform_Texture *GetPlatformData()  {return &m_pPlatformData;}
	int				  GetWidth(void)     {return m_iWidth;}
	int				  GetHeight(void)    {return m_iHeight;}

	void			  BeginScene(void);
	void			  EndScene(void);
	
private:
	char			 m_szFileName[MAX_FILENAME_LENGTH];
	bool			 m_bUsed;							// This texture slot is in use
	int				 m_iWidth;
	int				 m_iHeight;
	int				 m_iGroup;
	bool             m_bIsRenderTarget;
	TEXTUREFORMAT	 m_eFormat;
	uint32			 m_uFlags;
	Platform_Texture m_pPlatformData;
};


#endif //__BASE_TEXTURE_H__