/*! \file
 * Copyright (c) 2008
 * Project : Base
 * File    : Vector Header (Independent)
 * Author  : Dave Reed
 */

/*-Include-Once--------------------------------------------------------------*/

#if !defined(__BASE_MATH_VECTOR_H__)
#define __BASE_MATH_VECTOR_H__ /*!< \brief Include once define. */

/*-Classes-------------------------------------------------------------------*/

/* Forward class declarations */
class Matrix;

/*!
 * \brief The Vector class - Contains 4 components, but most members treat it as a 3D vector.
 */
class Vector
{
public:

	/*! \name Constructors */
	Vector( void );
	Vector( const float fX, const float fY, const float fZ, const float fW = 1.0f );
	Vector( const Vector &xV );

	/*! \name Assignment operators */
	Vector& operator  = ( const Vector &xV );
	Vector& operator += ( const Vector &xV );
	Vector& operator -= ( const Vector &xV );
	Vector& operator *= ( float fValue );
	Vector& operator /= ( float fValue );

	/*! \name Negation operator */
	Vector operator - () const;

	/*! \name Indexing operator */
	float &operator [] ( const int iElement );

	/*! \name Arithmetic operators */
	Vector operator + ( const Vector &xV ) const;
	Vector operator - ( const Vector &xV ) const;
	Vector operator * ( float fValue ) const;
	Vector operator / ( float fValue ) const;
	Vector operator * ( Matrix &xMtx ) const;

	/*! \name Comparison operators */
	bool operator == ( const Vector &xV ) const;
	bool operator != ( const Vector &xV ) const;
	bool ApproxEquals( const Vector &xV ) const;

	/*! \name Dot and Cross products (only 3-dimensional, ignores W component) */
	float  DotProduct  ( const Vector& xV ) const;
	Vector CrossProduct( const Vector& xV ) const;

	/*! \name Magnitude & Normalization functions */
	float MagnitudeSquared( void ) const;
	float Magnitude( void ) const;
	float Normalize( void );

	/*! \name Misc functions */
	Vector Lerp( const Vector &xTarget, const float fLerp ) const;
	void   CatmullRom( const Vector &p0, const Vector &p1, const Vector &p2, const Vector &p3, float s);

	/*! \name Data */
	float m_fX;
	float m_fY;
	float m_fZ;
	float m_fW;
};

#endif /* __BASE_MATH_VECTOR_H__ */
