

/*! \file
* Copyright (c) 2008
* Project : LD48 Framework
* File    : Game.cpp
* Author  : Dave Reed
*/

/*-Project-Includes----------------------------------------------------------*/



#include "Main.h"
#include "Game.h"
#include "Track.h"
#include "Racer.h"
#include "Record.h"
#include "Sound.h"

#include "Base/Render.h"
#include "Base/Blitter.h"
#include "Base/Texture.h"
#include "Base/Input.h"
#include "Base/Font.h"
#include "Base/PostProcess.h"

bool g_bEnablePP = true;

Font*		 Game::m_pMainFont			= 0;
Track		 Game::m_Track;
neSimulator *Game::m_pSimulator			= 0;
int          Game::m_eGameState			= 0;

Racer       *g_pPlayer = 0;
Texture     *g_pLogo   = 0;

int          g_iTrack = 0;
int          g_iLevel = 0;

float        g_aBestTimes[10][10];
float        g_fRaceTime;
bool		 g_bRaceCompleted = false;
float        g_fRaceEndTimer = 0.0f;
bool		 g_bNewBestTime = false;
float        g_fCheckpointTimer = 0.0f;

RecordBuffer *g_pCurrentReplay = NULL;
RecordBuffer *g_pBestReplay = NULL;

static int m_eGameState;


bool IsLevelOpen(int iLevel)
{	
	//return true;

	if (iLevel==0) return true;

	for(int i=0; i<10; i++)
	{
		if (g_aBestTimes[iLevel-1][i]<0.0f)
		{
			return false;
		}
	}

	return true;
}

void Game::Start(void)
{

}



void Game::Initialize(void)
{	
	m_pMainFont = new Font("cg36", 0, 4);	
	m_Track.Init(g_iLevel, g_iTrack);
	m_eGameState = GAMESTATE_TRACKSELECT;
	ResetPhysics();

	g_pLogo = Texture::LoadTexture("logo.png");

	Sound::Init();

	LoadBestTimes();
}

void Game::LoadBestTimes(void)
{
	for(int i=0;i<10;i++)
		for(int j=0;j<10;j++)
		{
			g_aBestTimes[i][j] = -1.0f;
		}

	FILE *f = fopen("data\\saves\\besttimes.dat", "rb" );
	if (f)
	{
		fread(g_aBestTimes, sizeof(float)*100, 1, f);
		fclose(f);
	}
	else
	{
		SaveBestTimes();
	}
}

void Game::SaveBestTimes(void)
{

	FILE *f = fopen("data\\saves\\besttimes.dat", "wb");
	if (f)
	{
		fwrite(g_aBestTimes, sizeof(float)*100, 1, f);
		fclose(f);
	}
}








void CollisionCallback(neCollisionInfo& collisionInfo)
{
	if (g_pPlayer)
	{
		g_pPlayer->CollisionCallback(collisionInfo);
	}
}


void Game::ResetPhysics()
{
	if (m_pSimulator)
	{
		neSimulator::DestroySimulator(m_pSimulator);
		m_pSimulator = NULL;
	}

	// creat the physics simulation
	neSimulatorSizeInfo sizeInfo;

	sizeInfo.rigidBodiesCount = 50;
	sizeInfo.animatedBodiesCount = 10; 
	sizeInfo.geometriesCount = 50;
	sizeInfo.overlappedPairsCount = 20;
	sizeInfo.rigidParticleCount = 0;
	sizeInfo.terrainNodesStartCount = 100000;
	
	neV3 gravity;
	gravity.Set(0.0f, -40.0f, 0.0f);
	m_pSimulator = neSimulator::CreateSimulator(sizeInfo, NULL, &gravity);
	m_pSimulator->SetMaterial(0,0,0);
	m_pSimulator->SetCollisionCallback(CollisionCallback);
	neCollisionTable *Coltable = m_pSimulator->GetCollisionTable();
	Coltable->Set (-1,0,neCollisionTable::RESPONSE_IMPULSE_CALLBACK); 

	m_Track.BuildPhysicsMesh(m_pSimulator);
	    
}


void Game::StartRace()
{

	if (g_pPlayer)
	{
		delete g_pPlayer;
		g_pPlayer = 0;
	}
	ResetPhysics();
	g_pPlayer = new Racer(m_pSimulator, Vector(0,1.1f,10));
	g_fRaceTime = -3.0f;

	g_bRaceCompleted = false;
	g_fRaceEndTimer = 0.0f;
	g_bNewBestTime = false;
	g_fCheckpointTimer = 0.0f;
	m_eGameState = GAMESTATE_PLAYING;

	m_Track.ResetCheckpoints();

	if (g_pCurrentReplay) delete g_pCurrentReplay;
	g_pCurrentReplay = new RecordBuffer(30*60*10);

	if (g_pBestReplay) delete g_pBestReplay;
	g_pBestReplay = new RecordBuffer(30*60*10);
	g_pBestReplay->Load(g_iLevel,g_iTrack);
}

void FormatTime(char *szBuf, float fTime)
{
	if (fTime<0.0f)
	{
		strcpy(szBuf,"-:--:--");
	}
	else
	{
		int   iSeconds = (int)fTime;
		float fFraction = fmodf(fTime, 1.0f);

		int  iMinutes = iSeconds/60;
		iSeconds -= (iMinutes*60);

		int iHundredths = (int)(fFraction*100.0f);

		sprintf(szBuf,"%d:%02d:%02d", iMinutes, iSeconds, iHundredths);
	}
}



void Game::UpdateAndRender(float fTimeStep)
{

	Camera xCam;			
	float fAspect =  Render::GetAspectRatio();
	xCam.SetProjection(1.0f, fAspect, 0.1f, 2500.0f);
	xCam.SetUpVector(Vector(0,1,0));

	if (m_eGameState == GAMESTATE_TRACKSELECT)
	{
		float dist = 650;
		static float t =0;
		t+=fTimeStep;
		Vector vCtr = m_Track.GetCentre();
		vCtr.m_fY -= 100.0f;
		Vector vCam = vCtr + Vector(dist * Math::Sin(t*0.5f), 450, dist * Math::Cos(t*0.5f));
		xCam.SetLookAt(vCam, vCtr);
	}
	else
	{	
		Vector vCam,vCtr;
		g_pPlayer->GetCamVectors(vCam, vCtr);	
		xCam.SetLookAt(vCam, vCtr);
	}

	Camera::Set(xCam);

	RenderStates::SetCullMode(RS_CULL_CCW);
	RenderStates::SetZBufferMode(RS_ZBUFFER_LESSEQUAL);
	RenderStates::SetZWriteEnable(true);
	RenderStates::SetAlphaTest(true, 1);
	RenderStates::SetBlendMode(RS_BLEND_NORMAL);

	if (Input::ActionPressed(ACTION_BUTTONY))
	{
		g_bEnablePP=!g_bEnablePP;
	}

	if (g_bEnablePP) PostProcess::BeginScene();

	Viewport::Clear(0x00201018); 


	Vector vOldPlayerPos;
	if (m_eGameState == GAMESTATE_PLAYING)
	{
		m_Track.Update(fTimeStep);
		vOldPlayerPos = g_pPlayer->GetPos();
		g_pPlayer->Update(fTimeStep, (g_fRaceTime>=0.0f));
		g_pPlayer->Render();

		if (g_fRaceTime>0.0f)
		{
			if (g_pCurrentReplay)
			{
				g_pCurrentReplay->Record(g_pPlayer->GetPosOrient(), g_fRaceTime);
			}
			
			Matrix xPosOrient;

			if (g_pBestReplay)
			{
				if (g_pBestReplay->Playback(xPosOrient, g_fRaceTime))
				{
					 g_pPlayer->Render(&xPosOrient, true);
				}
			}
		}
	}

	m_Track.Render();


	if (m_eGameState == GAMESTATE_TRACKSELECT)
	{
		// Kills any left over looping sounds
		Sound::STFU();

		bool bTrackChanged=false;


		if (Input::ActionPressed(ACTION_LEFT))
		{
			if (g_iTrack>0) 
			{
				Sound::Play(SFX_MENUBLEEP2);
				g_iTrack--;
				bTrackChanged = true;
			}
			else Sound::Play(SFX_MENUBUZZ);
		}

		if (Input::ActionPressed(ACTION_RIGHT))
		{
			if (g_iTrack<9) 
			{
				Sound::Play(SFX_MENUBLEEP2);
				g_iTrack++;
				bTrackChanged = true;
			}
			else Sound::Play(SFX_MENUBUZZ);
		}


		if (Input::ActionPressed(ACTION_DOWN))
		{
			if (g_iLevel>0) 
			{
				Sound::Play(SFX_MENUBLEEP2);
				g_iLevel--;
				bTrackChanged = true;
			}
			else Sound::Play(SFX_MENUBUZZ);
		}

		if (Input::ActionPressed(ACTION_UP))
		{
			if (g_iLevel<9 && IsLevelOpen(g_iLevel)) 
			{
				Sound::Play(SFX_MENUBLEEP2);
				g_iLevel++;
				bTrackChanged = true;
			}
			else Sound::Play(SFX_MENUBUZZ);
		}

		if (bTrackChanged)
		{
			static int iSeed=1;
			iSeed++;
			m_Track.DeInit();
			m_Track.Init(g_iLevel, g_iTrack);
			ResetPhysics();	
		}	
	}


	if (Input::ActionPressed(ACTION_RESTART))
	{
		if ((g_fRaceTime<1.0f || g_bRaceCompleted) && m_eGameState==GAMESTATE_PLAYING)
		{
			Sound::Play(SFX_MENUBLEEP2);
			m_eGameState = GAMESTATE_TRACKSELECT;
		}
		else
		{
			if (IsLevelOpen(g_iLevel))
			{
				Sound::Play(SFX_MENUBLEEP2);
				StartRace();
			}
			else Sound::Play(SFX_MENUBUZZ);
		}		
	}


	if (g_bEnablePP) PostProcess::ApplyBloom();
	if (g_bEnablePP) PostProcess::EndScene();


	m_pSimulator->Advance(fTimeStep, 4, NULL);

	Render::Set2DVirtualCoords(720.0f);
	RenderStates::SetBlendMode(RS_BLEND_NORMAL);


	float ctr = Render::Get2DScreenCentreX();

	
	char tmp[80];
	sprintf(tmp, "Level %d/10", g_iLevel+1 );

	if (IsLevelOpen(g_iLevel))
		m_pMainFont->PrintString(tmp,Render::Get2DScreenRightX()-20.0f, 20.0f, ALIGN_TOP|ALIGN_RIGHT, 0xffffffff, 1.0f);
	else
		m_pMainFont->PrintString(tmp,Render::Get2DScreenRightX()-20.0f, 20.0f, ALIGN_TOP|ALIGN_RIGHT, 0xffff0000, 1.0f);
	
	sprintf(tmp, "Track %d/10", g_iTrack+1 );
	m_pMainFont->PrintString(tmp,20.0f, 20.0f, ALIGN_TOP|ALIGN_LEFT, 0xffffffff, 1.0f);
	Render::Reset2DTransform();



	if (m_eGameState == GAMESTATE_TRACKSELECT)
	{
		BlitRect2D r;
		r.fX[0] = ctr -200;
		r.fX[1] = ctr +200;
		r.fY[0] = -20;
		r.fY[1] = -20+200;
		r.fU[0] = r.fV[0] = 0;
		r.fU[1] = r.fV[1] = 1;
		r.uARGB = 0xffffffff;
		Blitter2D::BlitRects(1,&r, g_pLogo);

		m_pMainFont->PrintString("Left/Right selects track", 20, 55.0f, ALIGN_TOP|ALIGN_LEFT, 0xffffffff, 0.6f);
		m_pMainFont->PrintString("Up/Down selects level", Render::Get2DScreenRightX()-20.0f, 55.0f, ALIGN_TOP|ALIGN_RIGHT, 0xffffffff, 0.6f);

		char tmp2[16];
		FormatTime(tmp2, g_aBestTimes[g_iLevel][g_iTrack]);
		sprintf(tmp, "Best Time: %s", tmp2);
		
		if (g_aBestTimes[g_iLevel][g_iTrack]<0.0f)
			m_pMainFont->PrintString(tmp, ctr, 615.0f, ALIGN_TOP|ALIGN_CENTRE, 0xffff0000, 1.0f);
		else
			m_pMainFont->PrintString(tmp, ctr, 615.0f, ALIGN_TOP|ALIGN_CENTRE, 0xffffffff, 1.0f);

		m_pMainFont->PrintString("<Space> to Race!", ctr, 660.0f, ALIGN_TOP|ALIGN_CENTRE, 0xffffffff, 0.6f);


		if (!IsLevelOpen(g_iLevel))
		{
			sprintf(tmp, "Level Locked (Complete all Level %d tracks first!)", g_iLevel);
			m_pMainFont->PrintString(tmp, ctr, 360.0f, ALIGN_MIDDLE|ALIGN_CENTRE, 0xffff0000, 1.0f);
		}
	}
	else
	{

		if (g_fRaceTime<4.0f)
		{
			if (g_fRaceTime<0.0f)
				m_pMainFont->PrintString("<Space> to change track", ctr, 660.0f, ALIGN_TOP|ALIGN_CENTRE, 0xffffffff, 0.6f);
			else
				m_pMainFont->PrintString("<Space> to restart", ctr, 660.0f, ALIGN_TOP|ALIGN_CENTRE, 0xffffffff, 0.6f);
		}

		if (g_fRaceEndTimer<=0.0f)		
		{
			if (m_Track.TestCheckpoints(vOldPlayerPos, g_pPlayer->GetPos()))
			{
				Sound::Play(SFX_CHECKPOINT);
				g_fCheckpointTimer = 1.0f;
			};

			// Check for win
			if (m_Track.Finished())
			{
				Sound::Play(SFX_WIN);
				g_bRaceCompleted = true;
				g_fRaceEndTimer = 3.0f;
				g_fCheckpointTimer = 0.0f;

				// Update best time
				if (g_aBestTimes[g_iLevel][g_iTrack]<0.0f || g_fRaceTime<g_aBestTimes[g_iLevel][g_iTrack])
				{
					g_aBestTimes[g_iLevel][g_iTrack] = g_fRaceTime;
					g_bNewBestTime=true;
					SaveBestTimes();
					
					if (g_pBestReplay) delete g_pBestReplay;
					g_pBestReplay = g_pCurrentReplay;					
					g_pBestReplay->Save(g_iLevel,g_iTrack);
					g_pCurrentReplay = 0;
				}
				
			}			
			else
			{			
				// Check for failure
				if (g_pPlayer->GetPos().m_fY < m_Track.m_vBBMin.m_fY)
				{
					Sound::Play(SFX_FAIL);
					g_bRaceCompleted = false;
					g_fRaceEndTimer = 3.0f;
				}
			}
		}

		if (g_fRaceEndTimer<=0.0f)
		{
			g_fRaceTime+=fTimeStep;
		}
		else
		{
			g_fRaceEndTimer-=fTimeStep;
			if (g_fRaceEndTimer<=0.0f)
			{
				g_fRaceEndTimer = 0.0f;
				
				if (g_bRaceCompleted)
					m_eGameState = GAMESTATE_TRACKSELECT;
				else
					StartRace();
			}

			uint32 iAlpha = ((uint32)(((g_fRaceEndTimer)/3.0f)*255.0f))<<24;

			float fScale = ((3.0f-g_fRaceEndTimer)/3.0f)*4.0f;

			if (g_bRaceCompleted)
			{
				if (g_bNewBestTime)
				{
					m_pMainFont->PrintString("Finished!", ctr, 350.0f, ALIGN_CENTRE|ALIGN_BOTTOM, 0xffffff|iAlpha, fScale+2.0f);		
					m_pMainFont->PrintString("New Best Time!", ctr, 350.0f, ALIGN_CENTRE|ALIGN_TOP, 0xffff00|iAlpha, fScale+2.0f);		
				}
				else
				{
					m_pMainFont->PrintString("Finished!", ctr, 350.0f, ALIGN_CENTRE|ALIGN_MIDDLE, 0xffffff|iAlpha, fScale+2.0f);		
				}
			}
			else
			{
				m_pMainFont->PrintString("FAIL!", ctr, 350.0f, ALIGN_CENTRE|ALIGN_MIDDLE, 0xff0000|iAlpha, fScale+2.0f);		
			}
		}		

		FormatTime(tmp, (g_fRaceTime>0)?g_fRaceTime:0);	

		for(int i=0; i<strlen(tmp); i++)
		{
			char tmp3[2] = {0,0};
			tmp3[0] = tmp[i];

			float fX = (float)(i-3)*32.0f;
			m_pMainFont->PrintString(tmp3, ctr + fX, 20.0f, ALIGN_CENTRE|ALIGN_TOP, 0xffffffff, 1.75f);					
		}

		// Show best time

		if (m_Track.MissedCheckpoint())
		{
			m_pMainFont->PrintString("Checkpoint Missed!", ctr, 80.0f, ALIGN_CENTRE|ALIGN_TOP, 0xffff0000, 0.75f);		
		}
		else
		{		
			if (g_aBestTimes[g_iLevel][g_iTrack]>0.0f)
			{
				char tmp2[20];
				FormatTime(tmp2, g_aBestTimes[g_iLevel][g_iTrack]);
				sprintf(tmp,"Best: %s",tmp2);
				m_pMainFont->PrintString(tmp, ctr, 80.0f, ALIGN_CENTRE|ALIGN_TOP, 0xffffffff, 0.75f);		
			}
		}

		if (g_fCheckpointTimer>0.0f)
		{			
			uint32 iAlpha = ((uint32)(g_fCheckpointTimer*255.0f))<<24;

			sprintf(tmp, "Checkpoint %d/%d", m_Track.GetLastCheckpoint()-1, m_Track.GetNumCheckpoints()-2);
			m_pMainFont->PrintString(tmp, 20.0f, 700.0f, ALIGN_LEFT|ALIGN_BOTTOM, 0x00ffffff | iAlpha, 0.75f);		
			g_fCheckpointTimer-=fTimeStep;
		}

		if (g_fRaceTime<1.0f)
		{
			float fFrac = 1.0f - fmodf(g_fRaceTime+3.0f, 1.0f);
			
			static int iLastVal = -1;
			int iVal = (int)(g_fRaceTime+3.0f);

			if (iVal!=iLastVal)
			{
				Sound::Play(SFX_THREE + iVal);
			}

			iLastVal=iVal;
			
			uint32 iAlpha = ((uint32)(fFrac*255.0f))<<24;
			char *szTxt[4] = {"3","2","1","Go!"};
			m_pMainFont->PrintString(szTxt[iVal], ctr, 220.0f, ALIGN_CENTRE|ALIGN_MIDDLE, iAlpha|0xffffff, 1.75f + (3.0f*(1.0f-fFrac)));		
		}
	}
}


void Game::Shutdown(void)
{
	delete m_pMainFont;
}