	
#include "Main.h"
#include "Base/Input.h"
#include "Base/Render.h"
#include "Racer.h"

#include "Sound.h"



Racer::Racer(neSimulator *pSim, Vector &vPos)
{
	m_pSimulator = pSim;
	m_vPos = vPos;

	// Create the body
	m_pBody = m_pSimulator->CreateRigidBody();
	neGeometry  *pGeom = m_pBody->AddGeometry();
	pGeom->SetSphereDiameter(2.0f);
	m_pBody->SetMass(1.0f);
	m_pBody->UpdateBoundingInfo();	
	neV3 pos;
	pos.Set(m_vPos.m_fX,m_vPos.m_fY,m_vPos.m_fZ);
	m_pBody->SetPos(pos);

	m_fOrient = 0.0f;
	m_fRV=0.0f;

	m_vUpTarget = m_vUp = Vector(0,1,0);

	m_fExtraCamHeight =0.0f;	
	m_fAirborne = 0.0f;	
	m_bThrust=false;
}


Racer::~Racer()
{

}



void Racer::Update(float fTimeStep, bool bInControl)
{
	float f = fTimeStep * 30.0f;


	float fLerpScale = 2.0f;

	if (bInControl)
	{
		
		if (Input::ActionHeld(ACTION_UP) || Input::ActionHeld(ACTION_BUTTONX))
		{
			neV3 i;
			i.Set(m_vFwd.m_fX*f,m_vFwd.m_fY*f,m_vFwd.m_fZ*f);
			m_pBody->ApplyImpulse(i);

			fLerpScale = 1.0f;

			m_bThrust = true;
			Sound::Play(SFX_THRUSTLOOP);
		}
		else
		{
			m_bThrust = false;
			Sound::Stop(SFX_THRUSTLOOP);
		}

		if (Input::ActionHeld(ACTION_DOWN)  || Input::ActionHeld(ACTION_BUTTONA))
		{
			neV3 i;
			i.Set(-m_vFwd.m_fX*f*2.0f,-m_vFwd.m_fY*f*2.0f,-m_vFwd.m_fZ*f*2.0f);
			m_pBody->ApplyImpulse(i);
			fLerpScale = 3.0f;
			Sound::Play(SFX_BRAKELOOP);
		}
		else
		{
			Sound::Stop(SFX_BRAKELOOP);
		}

		if (Input::ActionHeld(ACTION_RIGHT))
		{
			m_fRV-= fTimeStep*20.0f;
			if (m_fRV< -1.5f) m_fRV = -1.5f;
			Sound::Play(SFX_TURNLOOPR);
		}
		else
		{
			Sound::Stop(SFX_TURNLOOPR);
		}


		if (Input::ActionHeld(ACTION_LEFT))
		{				
			m_fRV+= fTimeStep*20.0f;
			if (m_fRV> 1.5f) m_fRV = 1.5f;
			Sound::Play(SFX_TURNLOOPL);
		}
		else
		{
			Sound::Stop(SFX_TURNLOOPL);
		}

	}

	m_fOrient+= (m_fRV * fTimeStep);
	m_fRV= m_fRV*0.8f;

	Matrix xRot;
	xRot.SetRotationY(m_fOrient);
	m_vFwd = Vector(0,0,1) * xRot;

	neV3 v = m_pBody->GetVelocity();
	Vector vVel = Vector(v.X(), v.Y(), v.Z());	
	float fy = vVel.m_fY;
	vVel.m_fY=0;
	float fSpeed = vVel.Normalize();

	Vector vNewVel = m_vFwd * fSpeed;
	vNewVel = vNewVel.Lerp((vVel*fSpeed),  1.0f - (fLerpScale*fTimeStep));
	vNewVel.m_fY = fy;

	v.Set(vNewVel.m_fX, vNewVel.m_fY, vNewVel.m_fZ);
	m_pBody->SetVelocity(v);

	neV3 pos = m_pBody->GetPos();
	m_vPos = Vector(pos.X(), pos.Y(), pos.Z());
    

	if (m_fAirborne<0.15f)
	{
		m_vUp = m_vUp.Lerp(m_vUpTarget, fTimeStep*5.5f);		
	}
	else
	{
		m_vUpTarget = Vector(0,1,0);
	}

	if (m_fAirborne>0.35f)
	{
		Sound::Play(SFX_FALLLOOP);
	}
	else
	{
		Sound::Stop(SFX_FALLLOOP);
	}

	if (m_fAirborne>0.15f)
	{		
		m_fExtraCamHeight += (fTimeStep*5.0f);
		if (m_fExtraCamHeight>10.0f) m_fExtraCamHeight=10.0f;
	}
	else m_fExtraCamHeight *= (1.0f - (fTimeStep*2.0f));

	m_fAirborne+=fTimeStep;

	m_vRight = m_vUp.CrossProduct(m_vFwd);
	m_vRight.Normalize();
	m_vFwd   = m_vRight.CrossProduct(m_vUp);
	m_vFwd.Normalize();
	

}


void Racer::CollisionCallback(neCollisionInfo& collisionInfo)
{
	bool bUpdate=false;
	neV3 vN;

	if (collisionInfo.bodyA == (neByte*)m_pBody)
	{
	    vN = collisionInfo.collisionNormal;
		bUpdate=true;
	}

	if (collisionInfo.bodyB == (neByte*)m_pBody)
	{
		vN = collisionInfo.collisionNormal;
		vN.Set(-vN.X(), -vN.Y(), -vN.Z());
		bUpdate=true;
	}

	if (bUpdate)
	{
		
		m_vUpTarget.m_fX = vN.X();
		m_vUpTarget.m_fY = vN.Y();
		m_vUpTarget.m_fZ = vN.Z();
		
		if (m_fAirborne>0.5f)
		{
			Sound::Play(SFX_LANDING);
		}

		m_fAirborne=0.0f;
	}
}


void BlitQuad(Vector*pVerts, int a, int b, int c, int d, uint32 uCol)
{
	BlitQuad3D q;
	int iIdxs[4] = {a,b,c,d};

	for(int i=0; i<4; i++)
	{
		q.fX[i] = pVerts[iIdxs[i]].m_fX;
		q.fY[i] = pVerts[iIdxs[i]].m_fY;
		q.fZ[i] = pVerts[iIdxs[i]].m_fZ;
	}

	q.uARGB = uCol; 

	Blitter3D::BlitQuads(1, &q, NULL);

}

void Racer::Render(Matrix *pxPosOrient, bool bWire)
{
	Matrix mtxRacer, mtxIdent;
	mtxIdent.SetIdentity();
	mtxRacer.SetTranslation(m_vPos);
	mtxRacer.SetUpVector(m_vUp);
	mtxRacer.SetRightVector(m_vRight);
	mtxRacer.SetForwardVector(m_vFwd);
	
	Matrix mtxTmp;
	mtxTmp.SetRotation(Vector(0,0,1), m_fRV*0.25f);
	Matrix mtxResult;
	mtxResult = mtxTmp * mtxRacer;
		

	if (bWire)
	{
		mtxResult = *pxPosOrient; 
	}
	else
	{
		m_xPosOrient = mtxResult;
	}

	Render::SetWorldMatrix(mtxResult);


	Vector aVerts[6] =
	{
		Vector(-0.5f, 0.5f, 3) * 0.4f,
		Vector( 0.5f, 0.5f, 3) * 0.4f,
		Vector(-1, 1.5f,-3) * 0.4f,
		Vector( 1, 1.5f,-3) * 0.4f,
		Vector(-2, 0,-2.5f) * 0.4f,
		Vector( 2, 0,-2.5f) * 0.4f
	};

	if (!bWire)
	{
		BlitQuad(&aVerts[0], 0,1,3,2, 0xff606060);
		BlitQuad(&aVerts[0],0,0,2,4, 0xff404040);
		BlitQuad(&aVerts[0],1,1,5,3, 0xff404040);
		if (m_bThrust)  BlitQuad(&aVerts[0],2,3,5,4, 0xffffA030);
				   else BlitQuad(&aVerts[0],2,3,5,4, 0xffB08020);

		BlitQuad(&aVerts[0],1,0,4,5, 0xff202020);
	}
	else
	{
		uint32 uCol = 0xffffff00;
		Blitter3D::DrawLine(aVerts[0], aVerts[1], uCol);
		Blitter3D::DrawLine(aVerts[0], aVerts[2], uCol);
		Blitter3D::DrawLine(aVerts[2], aVerts[3], uCol);
		Blitter3D::DrawLine(aVerts[1], aVerts[3], uCol);
		Blitter3D::DrawLine(aVerts[0], aVerts[4], uCol);
		Blitter3D::DrawLine(aVerts[1], aVerts[5], uCol);
		Blitter3D::DrawLine(aVerts[2], aVerts[4], uCol);
		Blitter3D::DrawLine(aVerts[3], aVerts[5], uCol);
		Blitter3D::DrawLine(aVerts[4], aVerts[5], uCol);
	}
	Blitter3D::Flush();


	Vector vTmp = m_vPos;
	vTmp.m_fW = 1.0f/3.0f;
	Shaders::SetSharedParamVector4(SHADER_PARAM_EFFECTCONTROL, vTmp);


	Render::SetWorldMatrix(mtxIdent);
}


void Racer::GetCamVectors(Vector &vFrom, Vector &vAt)
{
	vAt = m_vPos + (m_vUp*2.0f) ;
	vFrom = m_vPos-(m_vFwd*10.0f);


	vFrom.m_fY+=2.75f + m_fExtraCamHeight;
}