

#include "Main.h"
#include "Base/Render.h"
#include "Record.h"

#define RECORDING_TIMESTEP (1/15.0f)


void RecordBuffer::Record(Matrix& xPosOrient, float fTime)
{
	bool bRecord = false;

	if (iNumFrames==0)
	{
		bRecord = true;
	}
	else
	{
		if (fTime>pFrames[iNumFrames-1].fTime+RECORDING_TIMESTEP)
		{
			bRecord = true;
		}
	}

	if (bRecord)
	{
		if (iNumFrames<(iMaxNumFrames-1))
		{	
			pFrames[iNumFrames].xPosOrient = xPosOrient;
			pFrames[iNumFrames].fTime   = fTime;
			iNumFrames++;
		}	
	}
}


bool RecordBuffer::Playback(Matrix &xPosOrient, float fTime)
{
	if (iNumFrames<1) return false;
	if (fTime<pFrames[0].fTime) return false;
	if (fTime>pFrames[iNumFrames-1].fTime) return false;

	// Search for it
	int iPos = iLastPos;
	if (fTime<pFrames[iPos].fTime || iPos+1>iNumFrames) iPos = 0;
		

	while (fTime>pFrames[iPos+1].fTime)
	{
		iPos++;		
	}

	iLastPos = iPos;
	

	xPosOrient = pFrames[iPos].xPosOrient;

	// Interpolate the recorded positions
	float fLerp = (fTime - pFrames[iPos].fTime) / (pFrames[iPos+1].fTime - pFrames[iPos].fTime);

	Vector vPos1 = pFrames[iPos].xPosOrient.GetTranslationVector();
	Vector vPos2 = pFrames[iPos+1].xPosOrient.GetTranslationVector();	
    vPos1 = vPos1.Lerp(vPos2, fLerp);
	xPosOrient.SetTranslationVector(vPos1);

	Vector vUp1 = pFrames[iPos].xPosOrient.GetUpVector();
	Vector vUp2 = pFrames[iPos+1].xPosOrient.GetUpVector();	
	vUp1 = vUp1.Lerp(vUp2, fLerp);
	vUp1.Normalize();
	xPosOrient.SetUpVector(vUp1);

	Vector vFwd1 = pFrames[iPos].xPosOrient.GetForwardVector();
	Vector vFwd2 = pFrames[iPos+1].xPosOrient.GetForwardVector();	
	vFwd1 = vFwd1.Lerp(vFwd2, fLerp);
	vFwd1.Normalize();
	xPosOrient.SetForwardVector(vFwd1);

	Vector vRight1 = pFrames[iPos].xPosOrient.GetRightVector();
	Vector vRight2 = pFrames[iPos+1].xPosOrient.GetRightVector();	
	vRight1 = vRight1.Lerp(vRight2, fLerp);
	vRight1.Normalize();
	xPosOrient.SetRightVector(vRight1);


	return true;
}


void RecordBuffer::Save(int iLevel, int iTrack)
{
	char szTmp[80];
	sprintf(szTmp, "data\\saves\\%d-%d.replay",iLevel, iTrack);

	FILE *f = fopen(szTmp, "wb");
	if (f)
	{
		fwrite((void *)&iNumFrames,4,1,f);
		fwrite(pFrames, sizeof(RecordFrame)*iNumFrames, 1, f);
		fclose(f);
	}
}

void RecordBuffer::Load(int iLevel, int iTrack)
{
	char szTmp[80];
	sprintf(szTmp, "data\\saves\\%d-%d.replay",iLevel, iTrack);

	FILE *f = fopen(szTmp, "rb");
	if (f)
	{
		fread((void *)&iNumFrames,4,1,f);
		fread(pFrames, sizeof(RecordFrame)*iNumFrames, 1, f);
		fclose(f);
	}
}