
#include "Main.h"
#include "Track.h"

#include "Platform_Shaders.h"

#define CHECKPOINT_RADIUS 25.0f

Texture *g_pShadowTexture = 0;

void Track::Init(int iLevel, int iTrack)
{
	m_iSeed = iTrack + (iLevel*10) + 0x808080; 

	m_iNumSegments = 25 + (iLevel*10); 

	m_pSegments = new TrackSegment[m_iNumSegments];

	m_vBBMin = m_vBBMax = Vector(0,0,0);

	srand(m_iSeed);

	float fLevelScale = ((float)iLevel)/10.0f;


	float fDefWidth = 15.0f - iLevel;
	if (fDefWidth<8.0f) fDefWidth = 8.0f;

	m_pSegments[0].fWidth0 = m_pSegments[0].fWidth1 = fDefWidth;

	int iNarrowSectionCount = 0;
	

	for(int i=0; i<m_iNumSegments; i++)
	{
		int j=i-1;
		if (j<0) j=0;
		TrackSegment *pPrevSeg = &m_pSegments[j];

		int iSteps = 8+(4*(rand()%6));

		int iTurn = rand()%100;		
		float fTurn =  ((float)(iTurn-50)) * (0.025f + fLevelScale * 0.015f);
		float fElev=0.0f;

		if ((rand()%100)<(30 - iLevel*5))
		{
			fTurn=0;
			fElev=0;
		}
		else
		{
			if ((rand()%100)< 70 - (iTrack*4) )
			{
				fTurn = 0;

				fElev =  ((float)(iTurn-50)) * ((0.025f + fLevelScale * 0.02f)* (float)iSteps);		
			}
			else
			{
				int iElev = rand()%100;		


				// No height change on corners until lv4
				if (iLevel>2)
				{
					fElev =  ((float)(iElev-50)) * (0.045f* (float)iSteps);		

					// Avoid excessive elevation on tight ones bits
					float fT = 2.0f* fabsf(fTurn);
					if (fT>1.0f)
					{
						fElev /= (fT);
					}
				}					
			}
		}



		// Flat straight start+end
		if (i<2 || i>=(m_iNumSegments-2))
		{
		    iSteps= 8;
			fTurn = 0.0f;
			fElev = 0.0f;
		}

	
		float fWidth = fDefWidth;

		if (iNarrowSectionCount>0 && i>3 && i<(m_iNumSegments-3))
		{
			iNarrowSectionCount--;

			float fDiv = 2.0f + (((float)(iLevel))/4);
			fWidth/=fDiv;
		}
		else
		{
			if (rand()%300 < (3+iLevel))
			{
				iNarrowSectionCount = 2+rand()%4;
			}
		}

		m_pSegments[i].Init( pPrevSeg, iSteps, fTurn, fElev, fWidth);

		
		// Update bounds
		if (m_pSegments[i].vBBMin.m_fX<m_vBBMin.m_fX) m_vBBMin.m_fX = m_pSegments[i].vBBMin.m_fX;
		if (m_pSegments[i].vBBMin.m_fY<m_vBBMin.m_fY) m_vBBMin.m_fY = m_pSegments[i].vBBMin.m_fY;
		if (m_pSegments[i].vBBMin.m_fZ<m_vBBMin.m_fZ) m_vBBMin.m_fZ = m_pSegments[i].vBBMin.m_fZ;
		if (m_pSegments[i].vBBMax.m_fX>m_vBBMax.m_fX) m_vBBMax.m_fX = m_pSegments[i].vBBMax.m_fX;
		if (m_pSegments[i].vBBMax.m_fY>m_vBBMax.m_fY) m_vBBMax.m_fY = m_pSegments[i].vBBMax.m_fY;
		if (m_pSegments[i].vBBMax.m_fZ>m_vBBMax.m_fZ) m_vBBMax.m_fZ = m_pSegments[i].vBBMax.m_fZ;
	}

	m_vBBCentre = (m_vBBMin+m_vBBMax) * 0.5f;


	g_pShadowTexture = Texture::LoadTexture("shadow.png", false);


	// Add some checkpoints
	// Always add first at start of seg1 and last at start of last segment
	
	int iCPStep=5;
	m_iNumCheckpoints = 2;

	for(int i=2; i<m_iNumSegments-2; i++)
	{
		if (((i-1)%iCPStep) == 0) m_iNumCheckpoints++;
	}


	m_pCheckpoints = new Checkpoint[m_iNumCheckpoints];

	int iCP=0;
	m_pCheckpoints[iCP].Setup(&m_pSegments[1], iCP);
	iCP++;
	for(int i=2; i<m_iNumSegments-2; i++)
	{
		if (((i-1)%iCPStep) == 0)
		{
			m_pCheckpoints[iCP].Setup(&m_pSegments[i], iCP);
			iCP++;
		}
	}
	m_pCheckpoints[iCP].Setup(&m_pSegments[m_iNumSegments-1],iCP);
	iCP++;




	// Create checkpoint mesh
	float fInnerRadius = 19.0f;
	float fOuterRadius = CHECKPOINT_RADIUS;

	for (int i=0; i<=64; i++)
	{
		Vertex3D *pVert1 = &m_CheckpointVerts[i*2];
		Vertex3D *pVert2 = &m_CheckpointVerts[i*2+1];

		float fAng = (((float)i) / 64.0f) * Math::Pi() * 2.0f;
		float fS = Math::Sin(fAng);
		float fC = Math::Cos(fAng);

		float u = ((float)(i)/64.0f) * 3.0f;

		pVert1->fX = fS * fOuterRadius;
		pVert1->fY = fC * fOuterRadius;
		pVert1->fZ = 0.0f;
		pVert1->fU = u;
		pVert1->fV = 0.0f;
		pVert1->uARGB = 0xffbbbbbb;

		pVert2->fX = fS * fInnerRadius;
		pVert2->fY = fC * fInnerRadius;
		pVert2->fZ = 0.0f;
		pVert2->fU = u;
		pVert2->fV = 1.0f;
		pVert2->uARGB = 0xfffbbbbbb;

	}

	m_pCheckpointTexture = Texture::LoadTexture("Checkpoint.png");
	m_pCheckpointTextureS = Texture::LoadTexture("Checkpoints.png");
	m_pCheckpointTextureF = Texture::LoadTexture("Checkpointf.png");


	m_fTime = 0.0f;
}


void Track::BuildPhysicsMesh(neSimulator *pSim)
{
	// Build the physics mesh
	int iTriCount = 0;
	int iVertCount = 0;
	for(int i=0; i<m_iNumSegments; i++)
	{
		iTriCount  += (10 * m_pSegments[i].iNumSteps);
		iVertCount += (6 * (m_pSegments[i].iNumSteps+1));
	}

	static neTriangleMesh xMesh;

	xMesh.vertexCount    = iVertCount;
	xMesh.triangleCount  = iTriCount;
	xMesh.triangles		 = new neTriangle[iTriCount];
	xMesh.vertices		 = new neV3[iVertCount];

	int iTri=0;
	int iVert=0;
	for(int i=0; i<m_iNumSegments; i++)
	{
		TrackSegment *pSeg = &m_pSegments[i];

		for(int j=0; j<=pSeg->iNumSteps;j++)
		{
			for(int k=0; k<6; k++)
			{
				xMesh.vertices[iVert+(j*6)+k].Set(pSeg->pVerts[(j*6)+k].fX,  pSeg->pVerts[(j*6)+k].fY,  pSeg->pVerts[(j*6)+k].fZ );
			}			
		}

		for(int j=0; j<pSeg->iNumSteps;j++)
		{
			for(int k=0; k<5; k++)
			{
				neTriangle *pxTri1 = &xMesh.triangles[iTri++];
				neTriangle *pxTri2 = &xMesh.triangles[iTri++];

				pxTri1->materialID = pxTri2->materialID = 0;
				pxTri1->flag       = pxTri2->flag = neTriangle::NE_TRI_TRIANGLE;

				pxTri1->indices[0] = iVert+(j*6)+k+0;
				pxTri1->indices[1] = iVert+(j*6)+k+1;
				pxTri1->indices[2] = iVert+(j*6)+k+6;

				pxTri2->indices[0] = iVert+(j*6)+k+1;
				pxTri2->indices[1] = iVert+(j*6)+k+7;
				pxTri2->indices[2] = iVert+(j*6)+k+6;
			}
		}

		iVert+= (6*(pSeg->iNumSteps+1));		
	}

	for(int i=0; i<m_iNumSegments; i++)
	{
		m_pSegments[i].BuildRenderMesh(this);

		if (i==0 || i==m_iNumSegments-1)
		{
			// start/finish grid - replaces normal texture
			m_pSegments[i].m_pTextures[2] = Texture::LoadTexture("trackgrid.png");
		}
	}


	pSim->SetTerrainMesh(&xMesh);
}


void Track::DeInit(void)
{
	for(int i=0; i<m_iNumSegments; i++)
	{
		m_pSegments[i].DeInit();
	}
	delete [] m_pSegments;
	m_pSegments    = 0;
	m_iNumSegments = 0;


	delete [] m_pCheckpoints;
	m_iNumCheckpoints = 0;
}



void Track::Render(void)
{
	for(int i=0; i<m_iNumSegments; i++)
	{
		m_pSegments[i].Render();
	}

	Blitter3D::Flush();



	RenderStates::SetBlendMode(RS_BLEND_ADD);
	RenderStates::SetZWriteEnable(false);
	RenderStates::SetCullMode(RS_CULL_NONE);

	// render the checkpoints
	Matrix xIdent;
	xIdent.SetIdentity();
	for(int i=0; i<m_iNumCheckpoints; i++)
	{		
		if (i==0) Shaders::SetTexture(SHADER_BLITTER3D, 0, m_pCheckpointTextureS);			
		else if (i==m_iNumCheckpoints-1) Shaders::SetTexture(SHADER_BLITTER3D, 0, m_pCheckpointTextureF);			
		else Shaders::SetTexture(SHADER_BLITTER3D, 0, m_pCheckpointTexture);			

		Matrix xMtx, xRot;
		xRot.SetRotationZ(m_fTime);
		xMtx = xRot * m_pCheckpoints[i].xPosOrient;

		Render::SetWorldMatrix(xMtx);
		Platform_Shaders::Begin(SHADER_BLITTER3D);
		Platform_Render::GetD3DDevice()->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 64*2, &m_CheckpointVerts, sizeof(Vertex3D));
		Platform_Shaders::End();
		Render::SetWorldMatrix(xIdent);
	}

	RenderStates::SetZWriteEnable(true);
	RenderStates::SetBlendMode(RS_BLEND_NORMAL);
	RenderStates::SetCullMode(RS_CULL_CCW);
}


void TrackSegment::SetVertex(Vertex3D *pV, Vector& p, float u, float v)
{
	pV->fX = p.m_fX;
	pV->fY = p.m_fY;
	pV->fZ = p.m_fZ;
	pV->fU = u;
	pV->fV = v;
	pV->uARGB = 0xffffff00;

	if (pV->fX<vBBMin.m_fX) vBBMin.m_fX = pV->fX;
	if (pV->fY<vBBMin.m_fY) vBBMin.m_fY = pV->fY;
	if (pV->fZ<vBBMin.m_fZ) vBBMin.m_fZ = pV->fZ;
	if (pV->fX>vBBMax.m_fX) vBBMax.m_fX = pV->fX;
	if (pV->fY>vBBMax.m_fY) vBBMax.m_fY = pV->fY;
	if (pV->fZ>vBBMax.m_fZ) vBBMax.m_fZ = pV->fZ;
}

void TrackSegment::Init(TrackSegment *pPrevSeg, int iSteps, float fTurn, float fElevation, float fTargetWidth)
{	

	iNumSteps = iSteps;
	pVerts    = new Vertex3D[6*(iSteps+1)];

	vDir1 = vDir0  = pPrevSeg->vDir1;
	vP1   = vP0    = pPrevSeg->vP1;	
	fTilt1 = fTilt0 = pPrevSeg->fTilt1;
	fWidth0 = pPrevSeg->fWidth1;
	fWidth1 = fTargetWidth;
	
	Matrix xRot;

	float fSteps = (float)iSteps;

	for(int i=0; i<=iSteps; i++)
	{
		// Calc track 'out vector' and normal. Normal is the up vector rotated about the track direction
		Vector vNorm = Vector(0,1,0);
		xRot.SetRotation(vDir1, fTilt1);
		
		vNorm = vNorm * xRot;
		Vector vRight = vDir1.CrossProduct(vNorm);

		Vertex3D *pV = &pVerts[i*6];

		float fLerp = ((float)(i))/((float)(iSteps));
		float fWidth = fWidth0 * (1.0f-fLerp)  + fWidth1 * fLerp;

		float fWidth3 = fWidth+1.00f;
		float fWidth2 = fWidth+1.00f;

		if (i==0)
		{
			vBBMin = vBBMax = vP1;
		}

		SetVertex(&pV[0], vP1 - (vRight * fWidth3) - (vNorm*0.5f), 0, 0);
		SetVertex(&pV[1], vP1 - (vRight * fWidth2) + (vNorm*0.75f), 0, 0);
		SetVertex(&pV[2], vP1 - (vRight * fWidth), 0, 0);
		SetVertex(&pV[3], vP1 + (vRight * fWidth), 0, 0);
		SetVertex(&pV[4], vP1 + (vRight * fWidth2) + (vNorm*0.75f), 0, 0);
		SetVertex(&pV[5], vP1 + (vRight * fWidth3) - (vNorm*0.5f), 0, 0);

		if (i!=iSteps)  // <- important, don't advance it after the last set of verts has been generated
		{
			// Rotate vDir1 to turn the track
			// Inc vP1 for elevation
			// Tweak width here if needed
			xRot.SetRotationY(fTurn/fSteps);
			vDir1 = vDir1 * xRot;
			vDir1.m_fY = 0.0f;
			vDir1.Normalize();
			
			vP1 += vDir1 * 3.0f;
		
			float t = 3.142f*((float)i)/fSteps;
			t = 1.0f-((1.0f+(Math::Cos(t)))* 0.5f);
			vP1.m_fY = vP0.m_fY + fElevation*t;
			
			//fTilt1 = Math::Sin(t*2.0f) * (fTurn * 1.0f);
			//	if (fTilt1>0.3f) fTilt1=0.3f;
		}
	}

	vBSCentre = (vBBMax+vBBMin) * 0.5f;
	fBSRadius = (vBBMax-vBSCentre).Magnitude();
}



void SetVert(Vertex3DLit &VD, Vertex3D &VS, float u, float v, Vector &vN, uint32 uCol)
{
	VD.fX = VS.fX;
	VD.fY = VS.fY;
	VD.fZ = VS.fZ;
	VD.uARGB = uCol;
	VD.fU = u;
	VD.fV = v;
	VD.fNX = vN.m_fX;
	VD.fNY = vN.m_fY;
	VD.fNZ = vN.m_fZ;
}


void TrackSegment::BuildRenderMesh(class Track *pTrack)
{


	m_pTextures[0] = Texture::LoadTexture("track1.png");
	m_pTextures[1] = Texture::LoadTexture("track1.png");
	m_pTextures[2] = Texture::LoadTexture("track0.png");
	m_pTextures[3] = Texture::LoadTexture("track1.png");
	m_pTextures[4] = Texture::LoadTexture("track1.png");
	m_pTextures[5] = Texture::LoadTexture("track0.png");

	// 6 strips, including underside
	m_iVertsPerStrip = ((iNumSteps+1) * 2); 
	int iNumVerts = 6 * m_iVertsPerStrip;

	pShadedVerts = new Vertex3DLit[iNumVerts];

	int iVtx=0;

	const uint32 StripCol[6] = {0xffffffff, 0xff404040, 0xff888888, 0xffffffff, 0xff404040, 0xff202020};

	for(int iStrip = 0; iStrip<6; iStrip++)
	{
		int iCol0 = iStrip;
		int iCol1 = (iStrip+1)%6;

		for(int iRow=0; iRow<=iNumSteps; iRow++)
		{
			Vector vNrm = Vector(0,1,0);
			Vertex3DLit& V0 = pShadedVerts[iVtx++];
			Vertex3DLit& V1 = pShadedVerts[iVtx++];

			Vertex3D& SV0 = pVerts[iRow*6 + iCol0];
			Vertex3D& SV1 = pVerts[iRow*6 + iCol1];

			float v = ((float)(iRow)) *0.25f;
			SetVert(V0, SV0, 0, v, vNrm, StripCol[iStrip]);
			SetVert(V1, SV1, 1, v, vNrm, StripCol[iStrip]);
		}
	}
	
}



void TrackSegment::DeInit()
{
	if (pVerts)
	{
		delete [] pVerts;
		pVerts = 0;
	}
}


void TrackSegment::Render()
{

	if (!Camera::GetCurrentCamera().GetFrustum().SphereInFrustum(vBSCentre, fBSRadius))
	{
		return;
	}

	


	

	for(int i=0; i<6; i++)
	{
		Shaders::SetTexture(SHADER_BLITTER3D_LIT, 0, m_pTextures[i]);	
		Shaders::SetTexture(SHADER_BLITTER3D_LIT, 1, g_pShadowTexture);	

		Platform_Shaders::Begin(SHADER_BLITTER3D_LIT);
		Platform_Render::GetD3DDevice()->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, iNumSteps*2, &pShadedVerts[m_iVertsPerStrip*i], sizeof(Vertex3DLit));
		Platform_Shaders::End();
	}

}


void Track::ResetCheckpoints()
{
	for(int i=0; i<m_iNumCheckpoints; i++)
	{
		m_pCheckpoints[i].bPassed = false;
	}
	
	m_iExpectedNextCheckpoint = 0;
	m_bMissedCheckpoint = false;
	m_bFinished = false;
}

bool Track::TestCheckpoints(Vector &v1, Vector &v2)
{
	bool bCP=false;

	for(int i=0; i<m_iNumCheckpoints; i++)
	{
		if (!m_pCheckpoints[i].bPassed)
		{
			// Are we in it's sphere?
			Vector vPos = m_pCheckpoints[i].xPosOrient.GetTranslationVector();

			Vector vTmp = vPos - v1;

			if ( (vTmp.Magnitude()) < (CHECKPOINT_RADIUS+2.0f))
			{
				// Have we crossed it's plane?
				Vector &vPlane = m_pCheckpoints[i].vPlane;
				float f1 = v1.DotProduct(vPlane) + vPlane.m_fW;
				float f2 = v2.DotProduct(vPlane) + vPlane.m_fW;

				// If signs are different, we've crossed the plane
				if (f1*f2 < 0.0f)
				{
					m_pCheckpoints[i].bPassed = true;

					// Is this the point we're looking for?
					if (i>m_iExpectedNextCheckpoint)
					{
						m_bMissedCheckpoint = true;
					}
					else
					{
						m_bMissedCheckpoint = false;
						if (i>0) bCP=true;
						if (i+1>m_iExpectedNextCheckpoint) m_iExpectedNextCheckpoint = i+1;
					}
				}
			}			
		}
	}


	m_bFinished = true;
	for(int i=0; i<m_iNumCheckpoints; i++)
	{
		if (!m_pCheckpoints[i].bPassed)
		{
			m_bFinished = false;
		}
	}

	return bCP;
}