

// Track class

#ifndef __TRACK_H__
#define __TRACK_H__

#include "Base/Math.h"
#include "Base/Blitter.h"
#include "Tokamak.h"

class TrackSegment
{
public:

	TrackSegment()
	{
		vP0     = vP1   = Vector(0,0,0);
		vDir0   = vDir1 = Vector(0,0,1);
		fWidth0 = fWidth1 = 15.0f;		
		fTilt0  = fTilt1 = 0.0f;
		fTurn   = 0.0f;
		fElevation = 0.0f;
		pVerts = 0;
	}

	void   Init(TrackSegment *pPrev, int iSteps, float fTurn, float fElevation, float fTargetWidth);
	void   DeInit(void);
	void   Render(void);
	
	

	void   BuildRenderMesh(class Track *pTrack);
	
	void   SetVertex(Vertex3D *pV, Vector& p, float u, float v);

	float  fElevation;
	int	   iNumSteps;	
	float  fTurn;

	Vector vP0,     vP1;
	Vector vDir0,   vDir1;
	float  fTilt0,  fTilt1;		 // (Rotation around vDir)
	float  fWidth0, fWidth1;	

	Vector vBBMin;
	Vector vBBMax;
	Vector vBSCentre;
	float  fBSRadius;

	Vertex3D *pVerts;			// 6 * (iNumSteps+1)

	Vertex3DLit *pShadedVerts;
	int			m_iVertsPerStrip;

	Texture* m_pTextures[6];

};



class Checkpoint
{

public:

	Checkpoint()
	{
		bActive = false;
		iIndex = -1;
	}

	void Setup(TrackSegment *pSeg, int iIdx)
	{
		bActive = true;
		iIndex = iIdx;

		xPosOrient.SetIdentity();
		xPosOrient.SetTranslationVector(pSeg->vP0 + Vector(0,4,0));

		Vector vFwd = pSeg->vDir0;

		vPlane = vFwd;
		vPlane.m_fW = - (vFwd.DotProduct(pSeg->vP0 + Vector(0,4,0)));

		Vector vUp  = Vector(0,1,0);
		Vector vRight;
		vRight = vUp.CrossProduct(vFwd);
		vRight.Normalize();

		xPosOrient.SetForwardVector(vFwd);
		xPosOrient.SetUpVector(vUp);
		xPosOrient.SetRightVector(vRight);
	}

	bool   bActive;
	int	   iIndex;
	Matrix xPosOrient;
	Vector vPlane;
	float  fRadius;
	bool   bPassed;
};


class Track
{

public:
	void Init(int iLevel, int iTrack);
	void DeInit(void);
	void Render(void);
	void Update(float fTimeStep) {m_fTime+=fTimeStep;};
	void BuildPhysicsMesh(neSimulator *pSim);
	void ResetCheckpoints();
	bool TestCheckpoints(Vector &v1, Vector &v2);

	Vector& GetCentre() {return m_vBBCentre;}


	Vector		  m_vBBMin;
	Vector		  m_vBBMax;
	Vector		  m_vBBCentre;

	bool MissedCheckpoint(void) {return m_bMissedCheckpoint;}
	bool Finished(void) {return m_bFinished;}

	int    GetNumCheckpoints() {return m_iNumCheckpoints;}
	int    GetLastCheckpoint() {return m_iExpectedNextCheckpoint;}

private:



	int			  m_iSeed;
	int			  m_iNumSegments;
	TrackSegment *m_pSegments;

	Vertex3D      m_CheckpointVerts[65*2];
	Texture      *m_pCheckpointTexture;
	Texture      *m_pCheckpointTextureS;
	Texture      *m_pCheckpointTextureF;

	int           m_iNumCheckpoints;
	Checkpoint   *m_pCheckpoints;
	int			  m_iExpectedNextCheckpoint;
	bool		  m_bMissedCheckpoint;
	bool		  m_bFinished;


	float         m_fTime;

};






#endif // __TRACK_H__